#!/bin/sh
# the next line restarts using wish
# exec wish8.0 "$0" "$@"

# debug level

set debug(watch) 0
set debug(layout) 0
set debug(events) 0

# we don't want to see any error messages if xpa is not available

set env(XPA_VERBOSITY) 0

# local scripts library

lappend auto_path .

# required packages

package require tclxpa
#package require iis
package require BLT
package require saotk
package require DS9
package require http
package require FTP

# let's start

set ds9(init) 1

# Set Application Name

if {"$argv0"=="ds9.tcl"} {
    set ds9(app) ds9
} else {
    set ds9(app) [file tail $argv0]
}

tk appname $ds9(app)
set ds9(title) "$ds9(app)"
set ds9(version) "1.8.7.3"

# First, look for help quest

ProcessHelp

# ds9 Variables
# General Note:
# ds9(frames) contains list of all frames
# ds9(active) contains list of active (show) frames
# active(<framename>) contains status show/hide for framename
# slave(<framename>) contains list of frames that are slaves to framename
# master(<framename>) contains the name of the master frame of framename

set ds9(top) .
set ds9(visual) ""
set ds9(depth) 8
set ds9(frames) ""
set ds9(active) ""
set ds9(lock) 0
set ds9(next) ""
set ds9(sequence) 0
set ds9(mode) pointer
set ds9(centerpixel) 1
set ds9(dialog) motif
set ds9(scanOnLoad) 0
set ds9(cursor) 1
set ds9(xpa) 0
set ds9(displaymode) single

set ds9(b1) 0
set ds9(b2) 0
set ds9(b3) 0

set ds9(sb1) 0
set ds9(sb2) 0
set ds9(sb3) 0

set ds9(internalpad) 2
set ds9(framegap) 4

set ds9(lastFitsName) ""
set ds9(lastURL) ""
set ds9(lastMosaicName) ""
set ds9(lastColormapName) ""
set ds9(lastMarkerName) "ds9.reg"
set ds9(lastTextName) "ds9.txt"
set ds9(lastTextSearch) ""
set ds9(lastTclName) ""

set ds9(fileProcs) {StdinFits ExternalFits CompressFits PackFits GzipFits}

set ds9(extFile) ".$ds9(app).file"
set ds9(extAnalysis) ".$ds9(app).analysis"
set ds9(analysisItem) 0

# Help Variables

set help(top) .help
set help(mb) .helpMB
set help(try) 10
set help(ref) "http://hea-www.harvard.edu/RD/ds9/doc/ref/ref.html"
set help(user) "http://hea-www.harvard.edu/RD/ds9/doc/user/user.html"
set help(email) "saord@cfa.harvard.edu"

# Pixel Table Variables

set pixel(top) .pixel
set pixel(mb) .pixelMB

# Cursor Variables

set cursor(save) ""
set cursor(count) 0
set cursor(speed) 512
set cursor(mult) 10

# Crosshair Variables

set crosshair(lock) none

# Menu Variables

set menu(tearoff) 1
set menu(selectcolor) red

# Frame Variables

set current(frame) ""
set current(cursor) {}
set current(squarePixels) 0

set current(zoom) 1
set current(rotate) 0
set current(orient) none
set current(scale) linear

blt::vector histX histY

# Scale Variables

set scale(mode) minmax
set scale(scope) local
set minmax(mode) scan
set user(low) 0
set user(high) 100

# Marker Variables

set marker(shape) circle
set marker(color) green
set marker(edit) 1
set marker(move) 1
set marker(rotate) 1
set marker(delete) 1
set marker(include) 1
set marker(fixed) 0
set marker(maxdialog) 10
set marker(font) system
set marker(fontSize) 10
set marker(fontStyle) normal
set marker(saveformat) ds9
set marker(savecoord) image
set marker(savecoordformat) degrees

# WCS Variables

set wcs(align) 0
set wcs(system) fk5
set wcs(format) hms

# Colorbar Variables

set colorbar(name) ""
set colorbar(view) 1
set colorbar(width) 512
set colorbar(height) 15

set colorbar(map) Grey
set colorbar(invert) 0
set colorbar(count) 1
set colorbar(lock) 0
set colorbar(private) false

# Info Panel Variables

set info(view) 1
set info(name) ""
set info(filename) 1
set info(object) 0
set info(minmax) 0
set info(wcs) 1
set info(detector) 0
set info(amplifier) 0
set info(physical) 1
set info(image) 1
set info(frame) 1

# Buttons Variables

set buttons(name) ""
set buttons(view) 1

# Canvas Variables

set canvas(name) ""
set canvas(width) 512
set canvas(height) 512
set canvas(auto) 1
set canvas(size) 512

# Panner Variables

set pan(name) ""
set pan(view) 1
set pan(size) 128

set pan(imagecompass) 1
set pan(wcscompass) 1
set pan(wcstanpoint) 0
set pan(mode) click

# Magnifier Variables

set mag(name) ""
set mag(view) 1
set mag(size) 128
set mag(mode) accurate
set mag(region) 1
set mag(cursor) 1
set mag(zoom) 4

# Graph Variables

set graph(size) 150
set graph(font) "*helvetica-medium-r-*-100-*"
set graph(marginX) 35
set graph(marginY) 25
set graph(xMin) 0
set graph(xMax) 10
set graph(yMin) 1
set graph(yMax) 100
set graph(spacer) ""

set graphHorz(name) ""
set graphHorz(view) 0
set graphHorz(grid) 1
set graphHorz(log) false
blt::vector graphHorzX graphHorzY

set graphVert(name) ""
set graphVert(view) 0
set graphVert(grid) 1
set graphVert(log) false
blt::vector graphVertX graphVertY

# Printing Variables

set ps(dest) printer
set ps(cmd) "lp"
set ps(fileName) "ds9.ps"
set ps(orient) portrait
set ps(color) rgb
set ps(scale) scaled
set ps(size) letter
set ps(level) 2
set ps(interpolate) 0
set ps(resolution) 72
set ps(width) ""
set ps(height) ""

# Tile Variables

set tile(mode) grid

# Blink Variables

set blink(interval) 1000
set blink(index) -1
set blink(id) 0

# FITS Bin Table Support

set block(function) sum
set block(factor) 1
set block(buffersize) 1024

# Contour

set contour(status) 1
set contour(color) green
set contour(smooth) 16
set contour(levels) 10
set contour(copy) ""
set contour(top) .ct
set contour(mb) .ctMB

# Examine Variables

set examine(mode) new
set examine(zoom) 4

# HTTP Variables

set http(log) 1

# FTP Variables

set ftp(log) 1

# Preference Variables

set prefs(file) "~/.$ds9(app).prefs"

# Load any preferences here, before we do any real work

catch {source $prefs(file)}

# Toplevel-- Start here.

# We want to withdraw the window til everything is ready to go
wm withdraw $ds9(top)

set t "SAOImage $ds9(title)"
wm title $ds9(top) "$t"
wm iconname $ds9(top) "$t"

# This solves this problem:
# 1. tk no longer changes the vert size, thus generating two update sequences
# wm geometry $ds9(top) 750x749
# wm minsize $ds9(top) 640 480

$ds9(top) configure -menu .menuBar

# we need to set certain variables for color before anything else

ProcessColorCommandLine

set ds9(visual) [winfo visual .]
set ds9(depth) [winfo depth .]

switch $ds9(visual)$ds9(depth) {
    pseudocolor8 {}
    truecolor8 {}
    truecolor16 {}
    truecolor24 {}
    default {BadVisualError}
}

# create our main frame

frame .ds9

# we want the canvas to always to grow to consume all available space

grid rowconfigure .ds9 2 -weight 1
grid columnconfigure .ds9 0 -weight 1
pack .ds9 -fill both -expand true

# Create Colorbar-- Create this first, so in case of a private colormap,
# gui colors will be allocated in the new colormap, not the default colormap

CreateColorbar

# Create Canvas

set canvas(name) [canvas .ds9.image -width $canvas(width) -height $canvas(height) \
	-bd 2 -relief groove -highlightthickness 0 -insertofftime 0]

# Create other parts of the display

CreateMenuBar
CreateInfoPanel
CreateButtons
CreateGraphs

# Make sure that the wm knows when to swap in the colormap (if needed)

wm colormapwindows . "$canvas(name)"

# Initialize the display

InitColorbar
InitPanner

# Set our current state of things

CreateColorMenu
UpdateInfoPanel
ChangeMode
LayoutView

# We need this, cause we have SetWatchCursor turned off, hense no idletasks
# for some reason, CreateFrame needs to have an idletasks before creating a new frame

update idletasks
CreateFrame

# ok, ready to show the window

wm deiconify $ds9(top)

# we do this now, so configure events doesn't run LayoutFrames

InitCanvas

# Init external File Formats
# we want this before processing the command line

InitExternalFile

# Init analysis file formats

InitAnalysisFile

# and process any command line items
# we want to see something before any fits files are loaded

ProcessCommandLine

# Initialize XPA
# we need to do this after parsing the command line to get the correct title

catch {InitXPA}

# Init IIS

#InitIIS

# Load other user tcl code

catch {source "~/.$ds9(app)"}

# ok, we're done

set ds9(init) 0

#set eventCount 0
#proc doTimer {} {
#    global eventCount

#    puts "Timer...$eventCount"
#    UpdateFrame
#    incr eventCount
#    after 500 doTimer
#}
#doTimer
