package provide DS9 1.0

proc EllipseDialog {frame id} {
    global marker

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuMarkerDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordEllipseCB

    # ellipse menu items

    set marker($frame,$id,radiusformat) pixels
    RadiusFormatMenuEllipseDialog $frame $mb $id CoordEllipseCB
    $mb add cascade -label Radius -menu $mb.radiusformat

    # Init ellipse variables

    EditEllipseCB $frame $id
    RotateEllipseCB $frame $id

    # ellipse specific callbacks

    $frame marker $id callback edit EditEllipseCB $frame
    $frame marker $id callback rotate RotateEllipseCB $frame
    $frame marker $id callback delete DeleteEllipseDialog $frame

    # Dialog

    CommonMarkerDialog $frame $id
    wm protocol $w WM_DELETE_WINDOW "CloseEllipseDialog $frame $id"

    # Radius

    label $w.ref.radiusTitle -text "Radius"
    entry $w.ref.radius1 -textvariable marker($frame,$id,radius1) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.radius2 -textvariable marker($frame,$id,radius2) \
	-width 13 -relief sunken -bd 2
    label $w.ref.radiusCoord -textvariable marker($frame,$id,radiusformat) \
	-relief groove -bd 2 -width 9 -padx 4

    label $w.ref.angleTitle -text "Angle"
    entry $w.ref.angle -textvariable marker($frame,$id,angle) \
	-width 13 -relief sunken -bd 2
    label $w.ref.angleUnit -relief groove -bd 2 -width 9 -padx 4 -text "degrees"

    grid $w.ref.radiusTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.radius1 -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.radius2 -row 1 -column 2 -padx 4 -sticky w
    grid $w.ref.radiusCoord -row 1 -column 3 -padx 4 -sticky w

    grid $w.ref.angleTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ref.angle -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.angleUnit -row 2 -column 3 -padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyEllipseDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseEllipseDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyEllipseDialog $frame $id"
}

proc RadiusFormatMenuEllipseDialog {frame mb id cb} {
    global menu
    global marker

    menu $mb.radiusformat -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.radiusformat add radiobutton -label "Pixels" \
	-variable marker($frame,$id,radiusformat) -value pixels \
	-command "$cb $frame $id"
    if {[$frame has wcs equatorial]} {
	$mb.radiusformat add radiobutton -label "Degrees" \
	    -variable marker($frame,$id,radiusformat) -value degrees \
	    -command "$cb $frame $id"
	$mb.radiusformat add radiobutton -label "ArcMin" \
	    -variable marker($frame,$id,radiusformat) -value arcmin \
	    -command "$cb $frame $id"
	$mb.radiusformat add radiobutton -label "ArcSec" \
	    -variable marker($frame,$id,radiusformat) -value arcsec \
	    -command "$cb $frame $id"
    }
}

proc ApplyEllipseDialog {frame id} {
    EditEllipseDialog $frame $id
    RotateEllipseDialog $frame $id
    ApplyMarkerDialog $frame $id
}

proc CloseEllipseDialog {frame id} {
    global marker

    unset marker($frame,$id,radius1)
    unset marker($frame,$id,radius2)
    unset marker($frame,$id,angle)

    $frame marker $id delete callback edit EditEllipseCB
    $frame marker $id delete callback rotate RotateEllipseCB
    $frame marker $id delete callback delete DeleteEllipseDialog
    DeleteMarkerCBs $frame $id

    DeleteMarkerDialog $frame $id
}

proc DeleteEllipseDialog {frame id} {
    global marker

    unset marker($frame,$id,radius1)
    unset marker($frame,$id,radius2)
    unset marker($frame,$id,angle)
    unset marker($frame,$id,radiusformat)

    set mb ".mb$id"
    destroy $mb.radiusformat

    DeleteMarkerDialog $frame $id
}

proc EditEllipseDialog {frame id} {
    global marker

    $frame marker $id ellipse radius \
	$marker($frame,$id,radius1) $marker($frame,$id,radius2) \
	$marker($frame,$id,coord) $marker($frame,$id,radiusformat)
}

proc RotateEllipseDialog {frame id} {
    global marker

    $frame marker $id ellipse angle $marker($frame,$id,angle)
}

proc EditEllipseCB {frame id} {
    global marker

    set r [$frame get marker $id ellipse radius \
	       $marker($frame,$id,coord) $marker($frame,$id,radiusformat)]

    set marker($frame,$id,radius1) [lindex $r 0]
    set marker($frame,$id,radius2) [lindex $r 1]
}

proc RotateEllipseCB {frame id} {
    global marker

    set marker($frame,$id,angle) [$frame get marker $id ellipse angle]
}

proc CoordEllipseCB {frame id} {
    MoveMarkerCB $frame $id
    EditEllipseCB $frame $id
}

