package provide DS9 1.0

proc ButtonMarker {which x y} {
    global marker
    global current

    # see if we are on a handle

    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set marker(handle) [lindex $h 1]

    if {$marker(handle)} {
	$which marker $id edit begin $marker(handle)
	set marker(motion) beginEdit
	return
    }

    # else, see if we are on a segment of a polygon

    set h [$which get marker polygon segment $x $y]
    set id [lindex $h 0]
    set segment [lindex $h 1]
    if {$segment} {
	$which marker $id create polygon vertex $segment $x $y
	$which marker $id edit begin $marker(handle)
	set marker(handle) [expr 4+$segment+1]
	set marker(motion) beginEdit
	return
    }

    # else, see if we are on a marker

    if {[$which get marker id $x $y]} {
	$which marker select $x $y
	$which marker move begin $x $y
	set marker(motion) beginMove
	UpdateMarkerMenu
	return
    }

    # see if any markers are selected

    if {[$which get marker select number]>0} {
	$which marker unselect all
	set marker(motion) none
	return
    }

    # else, create a marker

    set marker(handle) 0
    set marker(motion) none

    set cmd "$which marker create $marker(shape) $x $y"
    switch $marker(shape) {
	circle {set cmd "$cmd 1"}
	ellipse {set cmd "$cmd 1 1 0"}
	rectangle {set cmd "$cmd 1 1 0"}
	polygon {set cmd "$cmd 1 1"}
	line {set cmd "$cmd $x $y"}
	text {
	    set txt "Region"
	    set r [EntryDialog "Text Region" "Enter Text:" 40 txt]
	    if {$r == 1 && $txt != {}} {
		set cmd "$cmd text = \{\"$txt\"\}"
	    } else {
		return
	    }
	}
	ruler {set cmd "$cmd $x $y"}
    }
    set cmd "$cmd color = $marker(color)"
    set cmd "$cmd font = \{\"$marker(font) $marker(fontSize) $marker(fontStyle)\"\}"
    set cmd "$cmd edit = $marker(edit) \
			 move = $marker(move) \
			 rotate = $marker(rotate) \
			 delete = $marker(delete) \
                         include = $marker(include) \
			 fixed = $marker(fixed)"

    $which marker unselect all
    set marker(id) [eval $cmd]
    set marker(motion) beginCreate
    set marker(createx) $x
    set marker(createy) $y

    switch $marker(shape) {
	circle -
	ellipse -
	rectangle -
	polygon {
	    set marker(handle) 1
	    $which marker $marker(id) edit begin $marker(handle)
	}
	line -
	ruler {
	    set marker(handle) 2
	    $which marker $marker(id) edit begin $marker(handle)
	}
    }
}

proc ShiftMarker {which x y} {
    global current
    global marker

    # see if we are on a handle

    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set marker(handle) [lindex $h 1]

    if {$marker(handle)} {
	$which marker $id rotate begin
	set marker(motion) beginRotate
	return
    }

    # else, see if we are on a marker

    if {[$which marker select shift $x $y]} {
	UpdateMarkerMenu
	$which marker move begin $x $y
	set marker(motion) beginMove
	return
    }

    # else, start a region select

    $which region select begin $x $y
    set marker(motion) shiftregion
}

proc MotionMarker {which x y} {
    global current
    global marker

    switch $marker(motion) {
	none {}

	beginCreate -
	create {
	    $which marker edit motion $x $y $marker(handle)
	    set marker(motion) create
	}

	beginMove -
	move {
	    $which marker move motion $x $y
	    set marker(motion) move
	}

	beginEdit -
	edit {
	    $which marker edit motion $x $y $marker(handle)
	    set marker(motion) edit
	}

	beginRotate -
	rotate {
	    $which marker rotate motion $x $y $marker(handle)
	    set marker(motion) rotate
	}

	region -
	shiftregion {$which region select motion $x $y}
    }
}

proc ReleaseMarker {which x y} {
    global current
    global marker

    switch $marker(motion) {
	none {}
	beginCreate {
	    # the user has just clicked, so resize to make visible or delete
	    # assumes marker(id) from create
	    $which marker edit end
	    switch [$which get marker $marker(id) type] {
		circle {
		    $which marker $marker(id) circle radius 20 image
		}
		ellipse {
		    $which marker $marker(id) ellipse radius 40 20 image
		}
		rectangle {
		    $which marker $marker(id) rectangle size 80 40 image
		}
		polygon -
		line -
		ruler {$which marker $marker(id) delete}
	    }
	}
	create {
	    $which marker edit end

	    # determine if this is an accident and just create the default
	    set diffx [expr $x-$marker(createx)]
	    set diffy [expr $y-$marker(createy)]
	    if {[expr sqrt($diffx*$diffx + $diffy*$diffy)]<2} {
		switch [$which get marker $marker(id) type] {
		    circle {
			$which marker $marker(id) circle radius 20 image
		    }
		    ellipse {
			$which marker $marker(id) ellipse radius 40 20 image
		    }
		    rectangle {
			$which marker $marker(id) rectangle size 80 40 image
		    }
		    polygon -
		    line -
		    ruler {$which marker $marker(id) delete}
		}
	    }
	}
	beginMove -
	beginRotate {}
	beginEdit {}
	move {$which marker move end}
	edit {$which marker edit end}
	rotate {$which marker rotate end}
	region {$which region select end}
	shiftregion {$which region select shift end}
    }
    unset marker(motion)
    unset marker(handle)
}

proc DoubleMarker {which x y} {
    set id [$which get marker id $x $y]

    if {$id} {
	MarkerDialog $which $id
    }
}

proc DeleteMarker {which x y} {
    global current

    # see if we are on a polygon
    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set handle [lindex $h 1]

    if {[$which get marker $id type] == "polygon"} {
	if {$handle > 4} {
	    # count vertex from 0
	    $which marker $id delete polygon vertex [expr $handle-5]
	} else {
	    # delete polygon
	    $which marker delete
	}
    } else {
	# delete marker
	$which marker delete
    }
}

proc CursorMarker {which x y cursor} {
    global ds9
    global marker

    # are we over any selected marker handles?
    # remember, handles are outside of a marker

    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set handle [lindex $h 1]
    if {$handle} {
	if {$handle < 5} {
	    # edit/rotate handle
	    SetCursor $cursor
	} else { 
	    # polygon vertex
	    SetCursor dotbox
	}
	return
    }

    # else, see if we are on a segement of a polygon

    set h [$which get marker polygon segment $x $y]
    if {[lindex $h 0]} {
	SetCursor draped_box
	return
    }

    # are we over a marker?

    if {[$which get marker select $x $y]} {
	SetCursor fleur
	return
    }

    # else, set no cursor

    SetCursor {}
}

proc FrontMarker {} {
    global current
    
    if {$current(frame) != ""} {
	$current(frame) marker move front
    }
}

proc BackMarker {} {
    global current
    
    if {$current(frame) != ""} {
	$current(frame) marker move back
	$current(frame) marker unselect all
    }
}

proc SelectAllMarker {} {
    global current
    
    if {$current(frame) != ""} {
	$current(frame) marker select all
    }
}

proc UnselectAllMarker {} {
    global current
    
    if {$current(frame) != ""} {
	$current(frame) marker unselect all
    }
}

proc DeleteAllMarker {} {
    global current
    
    if {$current(frame) != ""} {
	$current(frame) marker delete all
    }
}

proc ChangeMarkerColor {} {
    global current
    global marker
    
    if {$current(frame) != ""} {
	$current(frame) marker color $marker(color)
    }
}

proc ChangeMarkerProp {prop} {
    global current
    global marker

    if {$current(frame) != ""} {
	$current(frame) marker property $prop $marker($prop)
    }
}

proc ChangeMarkerFont {} {
    global current
    global marker

    if {$current(frame) != ""} {
	$current(frame) marker font \
	    \"$marker(font) $marker(fontSize) $marker(fontStyle)\"
    }
}

proc UnselectAllMarkers {} {
    global ds9

    foreach f $ds9(frames) {
	$f marker unselect all
    }
}

proc UpdateMarkerMenu {} {
    global ds9
    global current
    global marker
    global buttons

    if {$current(frame) != ""} {
	.menuBar entryconfig Region -state normal
	$buttons(name).major.region configure -state normal

	switch $ds9(mode) {
	    pointer {
		if {[$current(frame) get marker select number] == 1} {
		    set marker(color) [$current(frame) get marker color]
		    set marker(edit) [$current(frame) get marker property edit]
		    set marker(move) [$current(frame) get marker property move]
		    set marker(rotate) \
			[$current(frame) get marker property rotate]
		    set marker(delete) \
			[$current(frame) get marker property delete]
		    set marker(include) \
			[$current(frame) get marker property include]
		    set marker(fixed) \
			[$current(frame) get marker property fixed]

		    set f [$current(frame) get marker font]

		    set marker(font) [lindex $f 0]
		    set marker(fontSize) [lindex $f 1]
		    set marker(fontStyle) [lindex $f 2]
		}
	    }
	}
    } else {
	.menuBar entryconfig Region -state disabled
	$buttons(name).major.region configure -state disabled
    }
}

proc UpdateMarkerFormatMenu {} {
    global marker

    switch $marker(saveformat) {
	ds9 {
	    .menuBar.region.savecoord entryconfig "Equatorial B1950" \
		-state normal
	    .menuBar.region.savecoord entryconfig "Equatorial J2000" \
		-state normal
	    .menuBar.region.savecoord entryconfig "ICRS" -state normal
	    .menuBar.region.savecoord entryconfig "Galactic" -state normal
	    .menuBar.region.savecoord entryconfig "Ecliptic" -state normal
	    .menuBar.region.savecoord entryconfig "Detector" -state normal
	    .menuBar.region.savecoord entryconfig "Amplifier" -state normal
	    .menuBar.region.savecoord entryconfig "Physical" -state normal
	    .menuBar.region.savecoord entryconfig "Image" -state normal
	    .menuBar.region.savecoord entryconfig "Degrees" -state normal
	    .menuBar.region.savecoord entryconfig "HMS" -state normal
	}
	saotng {
	    if {$marker(savecoord) == "detector" ||
		$marker(savecoord) == "amplifier"} {
		set marker(savecoord) physical
	    }
	    .menuBar.region.savecoord entryconfig "Equatorial B1950" \
		-state normal
	    .menuBar.region.savecoord entryconfig "Equatorial J2000" \
		-state normal
	    .menuBar.region.savecoord entryconfig "ICRS" -state normal
	    .menuBar.region.savecoord entryconfig "Galactic" -state normal
	    .menuBar.region.savecoord entryconfig "Ecliptic" -state normal
	    .menuBar.region.savecoord entryconfig "Detector" -state disabled
	    .menuBar.region.savecoord entryconfig "Amplifier" -state disabled
	    .menuBar.region.savecoord entryconfig "Physical" -state normal
	    .menuBar.region.savecoord entryconfig "Image" -state normal
	    .menuBar.region.savecoord entryconfig "Degrees" -state normal
	    .menuBar.region.savecoord entryconfig "HMS" -state normal
	}
	saoimage {
	    set marker(savecoord) image
	    .menuBar.region.savecoord entryconfig "Equatorial B1950" \
		-state disabled
	    .menuBar.region.savecoord entryconfig "Equatorial J2000" \
		-state disabled
	    .menuBar.region.savecoord entryconfig "ICRS" -state disabled
	    .menuBar.region.savecoord entryconfig "Galactic" -state disabled
	    .menuBar.region.savecoord entryconfig "Ecliptic" -state disabled
	    .menuBar.region.savecoord entryconfig "Detector" -state disabled
	    .menuBar.region.savecoord entryconfig "Amplifier" -state disabled
	    .menuBar.region.savecoord entryconfig "Physical" -state disabled
	    .menuBar.region.savecoord entryconfig "Image" -state normal
	    .menuBar.region.savecoord entryconfig "Degrees" -state disabled
	    .menuBar.region.savecoord entryconfig "HMS" -state disabled
	}
	pros {
	    if {$marker(savecoord) == "detector" ||
		$marker(savecoord) == "amplifier"} {
		set marker(savecoord) "physical"
	    }
	    if {$marker(savecoord) == "icrs"} {
		set marker(savecoord) fk5
	    }
	    .menuBar.region.savecoord entryconfig "Equatorial B1950" \
		-state normal
	    .menuBar.region.savecoord entryconfig "Equatorial J2000" \
		-state normal
	    .menuBar.region.savecoord entryconfig "ICRS" -state disabled
	    .menuBar.region.savecoord entryconfig "Galactic" -state normal
	    .menuBar.region.savecoord entryconfig "Ecliptic" -state normal
	    .menuBar.region.savecoord entryconfig "Detector" -state disabled
	    .menuBar.region.savecoord entryconfig "Amplifier" -state disabled
	    .menuBar.region.savecoord entryconfig "Physical" -state normal
	    .menuBar.region.savecoord entryconfig "Image" -state normal
	    .menuBar.region.savecoord entryconfig "Degrees" -state normal
	    .menuBar.region.savecoord entryconfig "HMS" -state normal
	}
    }
}

proc OpenMarkerFile {} {
    global ds9
    LoadMarker [OpenFileDialog $ds9(top) ds9(lastMarkerName)]
}

proc LoadMarker {filename} {
    global current

    if {$current(frame) != ""} {
	if {$filename != ""} {
	    SetWatchCursor
	    if {[regexp -nocase {(.*).fits(.*)} $filename m1 m2 m3]} {
		if {$m3 == ""} {
		    set filename "$filename\[REGION\]"
		}
		if [catch {$current(frame) marker load fits $filename}] {
		    Error "Unable to load fits region file."
		}
	    } else {
		if [catch {$current(frame) marker load $filename}] {
		    Error "Unable to load region file."
		}
	    }
	    UnsetWatchCursor
	}
    }
}

proc SaveMarker {} {
    global ds9
    global current
    global marker

    if {$current(frame) != ""} {
	SetWatchCursor
	set fileName [SaveFileDialog $ds9(top) ds9(lastMarkerName)]

	if {$fileName != ""} {
	    $current(frame) marker save $fileName \
		$marker(saveformat) $marker(savecoord) $marker(savecoordformat)
	}
	UnsetWatchCursor
    }
}

proc InfoMarker {} {
    global current
    global marker

    if {$current(frame) != ""} {
	set l [$current(frame) get marker select]
	set i 0
	foreach d $l {
	    incr i
	    if {$i > $marker(maxdialog)} {
		return
	    }
	    MarkerDialog $current(frame) $d
	}
    }
}

proc MarkerDialog {frame id} {

    set type [$frame get marker $id type]

    switch -- $type {
	circle {CircleDialog $frame $id}
	ellipse {EllipseDialog $frame $id}
	rectangle {RectangleDialog $frame $id}
	polygon {PolygonDialog $frame $id}
	line {LineDialog $frame $id}
	text {TextDialog $frame $id}
	ruler {RulerDialog $frame $id}
	{circle point} -
	{box point} -
	{diamond point} -
	{cross point} -
	{x point} -
	{arrow point} -
	{boxcircle point} {PointDialog $frame $id}
    }
}

proc InitMarkerDialog {frame id} {
    global marker

    set w ".marker$id"
    set mb ".mb$id"

    # init marker variables

    set marker($frame,$id,coord) image
    set marker($frame,$id,format) degrees

    MoveMarkerCB $frame $id
    TextMarkerCB $frame $id
    ColorMarkerCB $frame $id
    PropertyMarkerCB $frame $id
    FontMarkerCB $frame $id

    # register callbacks

    $frame marker $id callback move MoveMarkerCB $frame
    $frame marker $id callback text TextMarkerCB $frame
    $frame marker $id callback color ColorMarkerCB $frame
    $frame marker $id callback property PropertyMarkerCB $frame
    $frame marker $id callback font FontMarkerCB $frame

    # define menus

    MBMarkerDialog $frame $mb
    ColorMenuMarkerDialog $frame $mb $id
#    PropMenuMarkerDialog $frame $mb $id  let each type define
#    CoordMenuMarkerDialog $frame $mb $id MoveMarkerCB let each type define
    FontMenuMarkerDialog $frame $mb $id
}

proc MBMarkerDialog {frame mb} {
    global menu

    menu $mb -tearoff 0 -selectcolor $menu(selectcolor)
    $mb add cascade -label Color -menu $mb.color
    $mb add cascade -label Properties -menu $mb.properties
    $mb add cascade -label Coordinates -menu $mb.coord
    $mb add cascade -label Font -menu $mb.font
}

proc ColorMenuMarkerDialog {frame mb id} {
    global menu
    global marker

    menu $mb.color -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.color add radiobutton -label "Black" \
	-variable marker($frame,$id,color) -value black \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "White" \
	-variable marker($frame,$id,color) -value white \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "Red" \
	-variable marker($frame,$id,color) -value red \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "Green" \
	-variable marker($frame,$id,color) -value green \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "Blue" \
	-variable marker($frame,$id,color) -value blue \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "Cyan" \
	-variable marker($frame,$id,color) -value cyan \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "Magenta" \
	-variable marker($frame,$id,color) -value magenta \
	-command "ColorMarkerDialog $frame $id"
    $mb.color add radiobutton -label "Yellow" \
	-variable marker($frame,$id,color) -value yellow \
	-command "ColorMarkerDialog $frame $id"
}

proc PropMenuMarkerDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Rotate" \
	-variable marker($frame,$id,rotate) \
	-command "PropertyMarkerDialog $frame $id rotate"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
    $mb.properties add checkbutton -label "Include/Exclude" \
	-variable marker($frame,$id,include) \
	-command "PropertyMarkerDialog $frame $id include"
    $mb.properties add checkbutton -label "Fixed in Size" \
	-variable marker($frame,$id,fixed) \
	-command "PropertyMarkerDialog $frame $id fixed"
}

proc CoordMenuMarkerDialog {frame mb id cb} {
    global menu
    global marker

    menu $mb.coord -tearoff 0 -selectcolor $menu(selectcolor)
    if {[$frame has wcs equatorial]} {
	$mb.coord add radiobutton -label "Equatorial B1950" \
	    -variable marker($frame,$id,coord) -value fk4 \
	    -command "$cb $frame $id"
	$mb.coord add radiobutton -label "Equatorial J2000" \
	    -variable marker($frame,$id,coord) -value fk5 \
	    -command "$cb $frame $id"
	$mb.coord add radiobutton -label "ICRS" \
	    -variable marker($frame,$id,coord) -value icrs \
	    -command "$cb $frame $id"
	$mb.coord add radiobutton -label "Galactic" \
	    -variable marker($frame,$id,coord) -value galactic \
	    -command "$cb $frame $id"
	$mb.coord add radiobutton -label "Ecliptic" \
	    -variable marker($frame,$id,coord) -value ecliptic \
	    -command "$cb $frame $id"
	$mb.coord add separator
    }
    if {[$frame has mosaic]} {
	$mb.coord add radiobutton -label "Amplifier" \
	    -variable marker($frame,$id,coord) -value amplifier \
	    -command "$cb $frame $id"
    }
    if {[$frame has mosaic] && [$frame has physical]} {
	$mb.coord add radiobutton -label "Detector" \
	    -variable marker($frame,$id,coord) -value detector \
	    -command "$cb $frame $id"
    }
    if {[$frame has physical]} {
	$mb.coord add radiobutton -label "Physical" \
	    -variable marker($frame,$id,coord) -value physical \
	    -command "$cb $frame $id"
    }
    $mb.coord add radiobutton -label "Image" \
	-variable marker($frame,$id,coord) -value image \
	-command "$cb $frame $id"
    if {[$frame has wcs equatorial]} {
	$mb.coord add separator
	$mb.coord add radiobutton -label "Degrees" \
	    -variable marker($frame,$id,format) -value degrees \
	    -command "$cb $frame $id"
	$mb.coord add radiobutton -label "HMS" \
	    -variable marker($frame,$id,format) -value hms \
	    -command "$cb $frame $id"
    }
}

proc FontMenuMarkerDialog {frame mb id} {
    global menu
    global marker

    menu $mb.font -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.font add radiobutton -label "Default" \
	-variable marker($frame,$id,font) -value system \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "Times" \
	-variable marker($frame,$id,font) -value times \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "Helvetica" \
	-variable marker($frame,$id,font) -value helvetica \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "Symbol" \
	-variable marker($frame,$id,font) -value symbol \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "Courier" \
	-variable marker($frame,$id,font) -value courier \
	-command "FontMarkerDialog $frame $id"
    $mb.font add separator
    $mb.font add radiobutton -label "9" \
	-variable marker($frame,$id,fontSize) -value 9 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "10" \
	-variable marker($frame,$id,fontSize) -value 10 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "12" \
	-variable marker($frame,$id,fontSize) -value 12 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "14" \
	-variable marker($frame,$id,fontSize) -value 14 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "18" \
	-variable marker($frame,$id,fontSize) -value 18 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "24" \
	-variable marker($frame,$id,fontSize) -value 24 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "30" \
	-variable marker($frame,$id,fontSize) -value 30 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "36" \
	-variable marker($frame,$id,fontSize) -value 36 \
	-command "FontMarkerDialog $frame $id"
    $mb.font add separator
    $mb.font add radiobutton -label "Plain" \
	-variable marker($frame,$id,fontStyle) -value normal \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "Bold" \
	-variable marker($frame,$id,fontStyle) -value bold \
	-command "FontMarkerDialog $frame $id"
    $mb.font add radiobutton -label "Italic" \
	-variable marker($frame,$id,fontStyle) -value italic \
	-command "FontMarkerDialog $frame $id"
}

proc CommonMarkerDialog {frame id} {
    global ds9
    global marker

    set w ".marker$id"
    set mb ".mb$id"

    # create window

    set type [string toupper "[$frame get marker $id type]"]
    toplevel $w
    wm title $w $type
    wm iconname $w $type
    wm group $w $ds9(top)

    $w configure -menu $mb

    # Dialog

    frame $w.basic -relief groove -borderwidth 2
    frame $w.ref -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.basic $w.ref $w.buttons -fill x -ipadx 4 -ipady 4

    # ID

    label $w.basic.idTitle -text "Id"
    label $w.basic.idValue -text "$id"

    # Text

    label $w.basic.textTitle -text "Text"
    entry $w.basic.textValue -textvariable marker($frame,$id,text) \
	-width 40 -relief sunken -bd 2

    grid $w.basic.idTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.basic.idValue -row 0 -column 1 -padx 4 -sticky w

    grid $w.basic.textTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.basic.textValue -row 1 -column 1 -padx 4 -sticky w -columnspan 3

    # Center

    label $w.ref.centerTitle -text "Center"
    entry $w.ref.centerX -textvariable marker($frame,$id,x) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.centerY -textvariable marker($frame,$id,y) \
	-width 13 -relief sunken -bd 2
    label $w.ref.centerCoord -relief groove -bd 2 -width 9 -padx 4 \
	-textvariable marker($frame,$id,coord)

    grid $w.ref.centerTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.ref.centerX -row 0 -column 1 -padx 4 -sticky w
    grid $w.ref.centerY -row 0 -column 2 -padx 4 -sticky w
    grid $w.ref.centerCoord -row 0 -column 3 -padx 4 -sticky w
}

proc ApplyMarkerDialog {frame id} {
    global marker

    $frame marker $id move to \
	$marker($frame,$id,x) $marker($frame,$id,y) $marker($frame,$id,coord)
    $frame marker $id text \"$marker($frame,$id,text)\"

    UpdateMarkerMenu
}

proc DeleteMarkerCBs {frame id} {
    $frame marker $id delete callback move MoveMarkerCB
    $frame marker $id delete callback text TextMarkerCB
    $frame marker $id delete callback color ColorMarkerCB
    $frame marker $id delete callback property PropertyMarkerCB
    $frame marker $id delete callback font FontMarkerCB
}

proc DeleteMarkerDialog {frame id} {
    global marker

    set w ".marker$id"
    set mb ".mb$id"

    unset marker($frame,$id,x)
    unset marker($frame,$id,y)
    unset marker($frame,$id,coord)
    unset marker($frame,$id,format)

    unset marker($frame,$id,text)
    unset marker($frame,$id,font)
    unset marker($frame,$id,fontSize)
    unset marker($frame,$id,fontStyle)

    unset marker($frame,$id,color)
    unset marker($frame,$id,edit)
    unset marker($frame,$id,move)
    unset marker($frame,$id,rotate)
    unset marker($frame,$id,delete)
    unset marker($frame,$id,include)
    unset marker($frame,$id,fixed)

    destroy $mb.color
    destroy $mb.properties
    destroy $mb.coord
    destroy $mb.font
    destroy $mb
    destroy $w
}

proc ColorMarkerDialog {frame id} {
    global marker

    $frame marker $id color $marker($frame,$id,color)
}

proc PropertyMarkerDialog {frame id prop} {
    global marker

    $frame marker $id property $prop $marker($frame,$id,$prop)
}

proc FontMarkerDialog {frame id} {
    global marker

    $frame marker $id font \
	\"$marker($frame,$id,font) $marker($frame,$id,fontSize) \
	$marker($frame,$id,fontStyle)\"
}

proc MoveMarkerCB {frame id} {
    global marker

    set c $marker($frame,$id,coord)
    if {$c == "fk4" | $c == "fk5" | $c == "icrs" | $c == "galactic" | \
	    $c == "ecliptic"} {
	set center [$frame get marker $id center \
			$marker($frame,$id,coord) $marker($frame,$id,format)]
    } else {
	set center [$frame get marker $id center $marker($frame,$id,coord)]
    }
    set marker($frame,$id,x) [lindex $center 0]
    set marker($frame,$id,y) [lindex $center 1]
}

proc ColorMarkerCB {frame id} {
    global marker

    set marker($frame,$id,color) [$frame get marker $id color]
}

proc TextMarkerCB {frame id} {
    global marker

    set marker($frame,$id,text) [$frame get marker $id text]
}

proc FontMarkerCB {frame id} {
    global marker

    set f [$frame get marker $id font]

    set marker($frame,$id,font) [lindex $f 0]
    set marker($frame,$id,fontSize) [lindex $f 1]
    set marker($frame,$id,fontStyle) [lindex $f 2]
}

proc PropertyMarkerCB {frame id} {
    global marker

    set marker($frame,$id,edit) [$frame get marker $id property edit]
    set marker($frame,$id,move) [$frame get marker $id property move]
    set marker($frame,$id,rotate) [$frame get marker $id property rotate]
    set marker($frame,$id,delete) [$frame get marker $id property delete]
    set marker($frame,$id,include) [$frame get marker $id property include]
    set marker($frame,$id,fixed) [$frame get marker $id property fixed]
}

# Markerlist

proc DisplayMarkerList {} {
    global mlDialog
    global current
    global marker

    set mlDialog(top) .ml
    set mlDialog(mb) .mlMB
    set mlDialog(txt) [$current(frame) marker list $marker(saveformat) \
			   $marker(savecoord) $marker(savecoordformat)]

    SimpleTextDialog mlDialog 80 20 "Region List" insert
}
