#include <ctype.h>
#include <string.h>
#include <iostream.h>

#include "tcl.h"
#include "iistcl.h"

extern "C" {
  int Iis_Init(Tcl_Interp* interp);
  int IisCmd(ClientData data, Tcl_Interp *interp, int argc, char* argv[]);
}


IIS* iis=NULL;

static char* dupstr(const char* str)
{
  char* copy;
  if (str) {
    copy=new char[strlen(str)+1];
    strcpy(copy,str);
  }
  else
    copy=NULL;

  return copy;
}

static char* toLower(char* str)
{
  char* ptr = str;
  while (*ptr) {
    *ptr = (char)(tolower(((int)(*ptr))));
    ptr++;
  }
  return str;
}

int Iis_Init(Tcl_Interp* interp) {

  // Define Package Name

  if (Tcl_PkgProvide(interp, "iis", "1.0") == TCL_ERROR)
    return TCL_ERROR;

  // Commands

  Tcl_CreateCommand(interp, "iis", IisCmd,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  //  iis = new IIS(interp);

  return TCL_OK;
}

int IisCmd(ClientData data, Tcl_Interp *interp, int argc, char* argv[])
{
  for (int i=1; i<argc; i++) {
    char* cmd = toLower(dupstr(argv[i]));
    if (!strncmp(cmd, "open", 4))
      return iis->open();
    else if (!strncmp(cmd, "close", 4))
      return iis->close();
    else {
      Tcl_AppendResult(interp, "iis: unknown command: ", argv[i], NULL);
      return TCL_ERROR;
    }

    delete cmd;
  }

  return TCL_OK;
}

IIS::IIS(Tcl_Interp* intp)
{
  interp = intp;
  fd = 0;
  chan = NULL;
  func = NULL;

  // Resources
  xim.autoscale = 0;         	         // is XY autoscaling enabled
  xim.antialias = 0;         	         // apply antialiasing when dezooming
  xim.tileFrames = 0;         	         // tile rather than overlay frames
  xim.highlightFrames = 0;               // highlight tiled frames
  xim.invert = 0;	         	 // use inverted colormap
  xim.def_config = 1;	         	 // default FB config
  xim.def_nframes = 0;	         	 // default number of frames
  xim.ncolors = DEF_NCOLORS;             // number of image pixel colors
  xim.tileBorder = DEF_TILE_BORDER;      // image border when tiling frames
  xim.borderColor = dupstr(DEF_BORDER_COLOR); // border color for tileFrames
  xim.gui = dupstr("default");        	 // GUI file name
  xim.imtoolrc = dupstr(FBCONFIG_2);     // imtoolrc file name
  xim.memModel = dupstr("fast");         // *FB memory model
  xim.userCMap1 = dupstr("none");        // user colormap file
  xim.userCMap2 = dupstr("none");        // user colormap file
  xim.userCMapDir1 = dupstr("none");     // user colormap directory
  xim.userCMapDir2 = dupstr(CMAPCONFIG); // user colormap directory
  xim.antialiasType = NULL;	         // type of antialiasing
  xim.printConfig = dupstr(PRINTCONFIG); // printer configuration file
  xim.input_fifo = dupstr(O_DEVNAME);    // *client's output, e.g. /dev/imt1o
  xim.output_fifo = dupstr(I_DEVNAME);   // *client's input, e.g. /dev/imt1i 
  xim.unixaddr = dupstr(DEF_UNIXADDR);   // *format for unix socket path
  xim.port = DEF_PORT;                   // *port for INET socket

  /* Internal state. */
  xim.obm = NULL;		// object manager
  for (int i=0; i<MAX_CLIENTS; i++)
    xim.chan[i].type = 0;
  xim.cursor_chan = NULL;	// *cursor mode channel
  xim.toplevel = NULL;		// dummy toplevel app shell
  xim.gt = NULL;		// *imagewin gterm-image widget
  xim.cb = NULL;		// colorbar gterm-image widget
  xim.gm_border = NULL;		// border marker for tileFrames
  xim.tileFramesList = 0;	// frames to be tiled (bitmask)
  xim.nTileFrames = 0;		// number of frames to be tiled
  xim.rop = 0;			// rasterop for mappings
  xim.display_frame = 0;	// currently displayed frame
  xim.df_p = NULL;		// *display frame descriptor
  //xim.frames[MAX_FRAMES];	// *array of frame descriptors
  xim.fb_configno = 0;		// *current config number
  xim.nframes = 0;		// *current number of frame bufs
  xim.width = 0;
  xim.height = 0;		// *current width, height
  //xim.fb_config[MAX_FBCONFIG]; // *fb config table
  xim.clientPrivate = NULL;	// used by imtool client code

  xim.psim = NULL;		// EPS image struct pointer
  xim.pcp = NULL;		// printer config pointer
  xim.flp = NULL;	        // load disk file pointer
  xim.fsp = NULL;		// save disk file pointer

  // and initialize
  xim_initialize(&xim, xim.def_config, xim.def_nframes, 1);
}

IIS::~IIS()
{
  if (xim.borderColor)
    delete xim.borderColor;
  if (xim.gui)
    delete xim.gui;
  if (xim.imtoolrc)
    delete xim.imtoolrc;
  if (xim.memModel)
    delete xim.memModel;
  if (xim.userCMap1)
    delete xim.userCMap1;
  if (xim.userCMap2)
    delete xim.userCMap2;
  if (xim.userCMapDir1)
    delete xim.userCMapDir1;
  if (xim.userCMapDir2)
    delete xim.userCMapDir2;
  if (xim.antialiasType)
    delete xim.antialiasType;
  if (xim.printConfig)
    delete xim.printConfig;
  if (xim.input_fifo)
    delete xim.input_fifo;
  if (xim.output_fifo)
    delete xim.output_fifo;
  if (xim.unixaddr)
    delete xim.unixaddr;
}

void IIS::eval(char* cmd)
{
  if (Tcl_Eval(interp, cmd) == TCL_ERROR)
    cerr << interp->result << endl;
}

int IIS::open()
{
  xim_iisopen(&xim);

  return TCL_OK;
}

int IIS::close()
{
  xim_iisclose(&xim);

  return TCL_OK;
}

