#include <stdlib.h>
#include <string.h>

#include "colorscaletrue16.h"
#include "util.h"

ColorScaleTrueColor16::ColorScaleTrueColor16(int s,
					     unsigned long red,
					     unsigned long green,
					     unsigned long blue)
  : ColorScale(s)
{
  colors_ = new unsigned char[s*2];

  int rs,gs,bs;
  unsigned short rm = decodeMask((unsigned short)red, &rs);
  unsigned short gm = decodeMask((unsigned short)green, &gs);
  unsigned short bm = decodeMask((unsigned short)blue, &bs);

  for (int i=0; i<s; i++) {
    unsigned short r = psColors_[i*3+2];
    unsigned short g = psColors_[i*3+1];
    unsigned short b = psColors_[i*3];
    unsigned short result = 0;
    result |= rs>0 ? ((r & rm) << rs) : ((r & rm) >> -rs);
    result |= gs>0 ? ((g & gm) << gs) : ((g & gm) >> -gs);
    result |= bs>0 ? ((b & bm) << bs) : ((b & bm) >> -bs);

    memcpy(colors_+i*2, &result, 2);
  }
}

ColorScaleTrueColor16::~ColorScaleTrueColor16()
{
  if (colors_)
    delete [] colors_;
}

LinearScaleTrueColor16::LinearScaleTrueColor16(int s, 
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count,
					       unsigned long red,
					       unsigned long green,
					       unsigned long blue) 
  : LinearScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, red, green, blue),
    ColorScale(s) {}

LogScaleTrueColor16::LogScaleTrueColor16(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count,
					 unsigned long red,
					 unsigned long green,
					 unsigned long blue) 
  : LogScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, red, green, blue),
    ColorScale(s) {}

SquaredScaleTrueColor16::SquaredScaleTrueColor16(int s,
						 unsigned short* indexCells,
						 unsigned char* colorCells,
						 int count,
						 unsigned long red,
						 unsigned long green,
						 unsigned long blue) 
  : SquaredScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, red, green, blue),
    ColorScale(s) {}

SqrtScaleTrueColor16::SqrtScaleTrueColor16(int s,
					   unsigned short* indexCells,
					   unsigned char* colorCells,
					   int count,
					   unsigned long red,
					   unsigned long green,
					   unsigned long blue) 
  : SqrtScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor16(s, red, green, blue),
    ColorScale(s) {}

