#include <stdlib.h>
#include <string.h>

#include "colorscaletrue24.h"
#include "util.h"

ColorScaleTrueColor24::ColorScaleTrueColor24(int s, int bpp, int msb) : ColorScale(s)
{
  switch (bpp) {
  case 3:
    colors_ = psColors_;
    manageColors = 0;
    break;

  case 4:
    {
      colors_ = new unsigned char[s*bpp];
      memset(colors_, 0, s*bpp);
      manageColors = 1;
      
      // if 4 bytes per pixel, we have non-used alpha channel
      // on big endian, alpha is at the lower end, so skip

      int offset = msb ? 1 : 0;
      for (int i=0; i<s; i++)
	memcpy(colors_+i*4+offset, psColors_+i*3, 3);
    }
    break;
  }
}

ColorScaleTrueColor24::~ColorScaleTrueColor24()
{
  if (manageColors && colors_)
    delete [] colors_;
}

LinearScaleTrueColor24::LinearScaleTrueColor24(int s, 
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count, int bpp, int msb) 
  : LinearScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor24(s, bpp, msb),
    ColorScale(s)
{}

LogScaleTrueColor24::LogScaleTrueColor24(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count, int bpp, int msb)
  : LogScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor24(s, bpp, msb),
    ColorScale(s) {}

SquaredScaleTrueColor24::SquaredScaleTrueColor24(int s,
						 unsigned short* indexCells,
						 unsigned char* colorCells,
						 int count, int bpp, int msb)
  : SquaredScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor24(s, bpp, msb),
    ColorScale(s) {}

SqrtScaleTrueColor24::SqrtScaleTrueColor24(int s,
					   unsigned short* indexCells,
					   unsigned char* colorCells,
					   int count, int bpp, int msb)
  : SqrtScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor24(s, bpp, msb),
    ColorScale(s) {}
