#include "contour.h"
#include "util.h"
#include "nan.h"

ContourPlot::ContourPlot(Frame* p, const char* c) : parent(p)
{
  contours = new List<Vertex>;
  colorName = dupstr(c);
  color_ = parent->getColor(colorName);
}

ContourPlot::ContourPlot(Frame* p, const char* c, List<Vertex>* cont) 
  : parent(p), contours(cont)
{
  colorName = dupstr(c);
  color_ = parent->getColor(colorName);
}

ContourPlot::~ContourPlot()
{
  if (contours)
    delete contours;

  if (colorName)
    delete colorName;
}

void ContourPlot::color(const char* c)
{
  if (colorName)
    delete colorName;

  colorName = dupstr(c);
  color_ = parent->getColor(colorName);
}

void ContourPlot::render()
{
  if (contours->head()) {
    XSetForeground(parent->display, parent->contourGC, color_);

    Vector u1 = contours->current()->vector;
    while (contours->next()) {
      Vector u2 = contours->current()->vector;
      if (u1[0] != DBL_MAX && u2[0] != DBL_MAX) {
	Vector v1 = u1 * parent->refToWidget;
	Vector v2 = u2 * parent->refToWidget;
	XDrawLine(parent->display, parent->basePixmap, parent->contourGC, 
		  (int)v1[0], (int)v1[1], (int)v2[0], (int)v2[1]);
      }

      u1 = u2;
    }
  }
}

void ContourPlot::ps(Frame::PSColorSpace mode)
{
  if (contours->head()) {
    char buf[256];

    {
      ostrstream str(buf,256);

      switch (mode) {
      case Frame::BW:
      case Frame::GRAY:
	psColorGray(colorName, str);
	str << " setgray";
	break;
      case Frame::RGB:
	psColorRGB(colorName, str);
	str << " setrgbcolor";
	break;
      case Frame::CMYK:
	psColorCMYK(colorName, str);
	str << " setcmykcolor";
	break;
      }
      str << endl << ends;

      Tcl_AppendResult(parent->interp, buf, NULL);
    }  
  
    // output stroke paths

    int first = 1;
    do {
      ostrstream str(buf,256);

      Vector u = contours->current()->vector;
      if (u[0] != DBL_MAX) {
	Vector v = u * parent->refToCanvas;
	if (first) {
	  str << "newpath " << endl
	      << v.TkCanvasPs(parent->canvas) << " moveto" << endl << ends;
	  first = 0;
	}
	else
	  str << v.TkCanvasPs(parent->canvas) << " lineto" << endl << ends;
      }
      else {
	str << "stroke" << endl << ends;
	first = 1;
      }

      Tcl_AppendResult(parent->interp, buf, NULL);
    } while (contours->next());
  }
}

List<Vertex>* ContourPlot::dup()
{
  List<Vertex>* dp = new List<Vertex>;

  if (contours->head())
    do
      dp->append(new Vertex(contours->current()->vector));
    while (contours->next());

  return dp;
}
