%{
#define YYPARSE_PARAM fr
#define FR ((Frame*)fr)
#define YYDEBUG 1

#define PASSTHRU_(x) {yyclearin; frPassThru(x);}

#include <math.h>
#include <string.h>

#include "frame.h"
#include "util.h"

extern int frlex(void);
extern void frerror(const char*);
extern void frPassThru(int);

static unsigned short defaultProps = Marker::SELECT |
	Marker::EDIT | Marker::MOVE | Marker::ROTATE | 
	Marker::DELETE | Marker::INCLUDE;
static unsigned short currentProps;
static int currentID;
static Vector currentVector;
static Vector currentVector2;
static char currentColor[16];
static char currentFont[32];
static char currentText[80];

static void setProps(unsigned short prop, int which);
%}

%union {
  float real;
  int integer;
  char str[256];
  void* ptr;
}

%type <real> numeric
%type <integer> yesno
%type <real> angle
%type <real> hmsCoord
%type <real> dmsCoord
%type <integer> precision
%type <integer> canvasCoord
%type <integer> coordSystem
%type <integer> coordSystemCanvas
%type <integer> coordFormat
%type <integer> wcsSystem
%type <integer> equatorial
%type <integer> wcsFormat
%type <integer> markerProperty
%type <integer> markerCallBack
%type <integer> markerFileFormat
%type <integer> pscolorspace

%token <real> REAL
%token <integer> INT
%token <str> STRING
%token <ptr> POINTER
%token <integer> HOUR
%token <integer> MINUTE
%token <real> SECOND

%token <real> DEGREE
%token <real> ARCMINUTE
%token <real> ARCSECOND

%token <str> HMSSTR

%token <real> RADIAN
%token <integer> PIXEL

%token ABOUT_
%token ACCURATE_
%token ALLOC_
%token ALIGN_
%token ALL_
%token AMPLIFIER_
%token ANGLE_
%token ARCMIN_
%token ARCSEC_
%token ARRAY_
%token ARROW_
%token AUTOMATIC_
%token AVERAGE_
%token B1950_
%token BACK_
%token BBOX_
%token BEGIN_
%token BLOCK_
%token BOX_
%token BOXCIRCLE_
%token BUFFER_
%token BW_
%token CALLBACK_
%token CANVAS_
%token CENTER_
%token CHANNEL_
%token CIRCLE_
%token CLEAR_
%token CLIP_
%token COLORMAP_
%token COLOR_
%token COLORS_
%token COLORSCALE_
%token COLORSPACE_
%token COMMAND_
%token CONTOUR_
%token COORDINATES_
%token COPY_
%token COUNT_
%token CURSOR_
%token CREATE_
%token CROSS_
%token CROSSHAIR_
%token CUT_
%token CMYK_
%token DATAMIN_
%token DEBUG_
%token DEGREES_
%token DEFAULT_
%token DELETE_
%token DETECTOR_
%token DIAMOND_
%token DISTANCE_
%token DS9_
%token EDIT_
%token ECLIPTIC_
%token ELLIPSE_
%token END_
%token EQUATORIAL_
%token ERASE_
%token EXT_
%token FACTOR_
%token FALSE_
%token FAST_
%token FILE_
%token FIT_
%token FITS_
%token FIXED_
%token FK4_
%token FK5_
%token FONT_
%token FRONT_
%token FULL_
%token FUNCTION_
%token GALACTIC_
%token GET_
%token GLOBAL_
%token GRAPHICS_
%token GRAY_
%token HANDLE_
%token HAS_
%token HEADER_
%token HEIGHT_
%token HIDE_
%token HIGH_
%token HIGHLITE_
%token HISTOGRAM_
%token HMS_
%token HORIZONTAL_
%token ICRS_
%token ID_
%token IIS_
%token IMAGE_
%token INCLUDE_
%token INFO_
%token INTEGER_
%token INTERPOLATE_
%token IRAFMIN_
%token J2000_
%token LEVEL_
%token LINE_
%token LINEAR_
%token LIST_
%token LOAD_
%token LOCAL_
%token LOG_
%token LOW_
%token MAGNIFIER_
%token MANUAL_
%token MARKER_
%token MASTER_
%token MINMAX_
%token MMAP_
%token MOSAIC_
%token MODE_
%token MOTION_
%token MOVE_
%token N_
%token NAME_
%token NAMES_
%token NEW_
%token NO_
%token NONE_
%token NUMBER_
%token OBJECT_
%token OFF_
%token ON_
%token ORIENT_
%token PAN_
%token PANNER_
%token PARAM_
%token PASTE_
%token PHYSICAL_
%token PIXELS_
%token POINT_
%token POLYGON_
%token POSTSCRIPT_
%token PROPERTY_
%token PROS_
%token RADIUS_
%token RECTANGLE_
%token REGION_
%token RESET_
%token RESOLUTION_
%token RGB_
%token ROTATE_
%token RULER_
%token SAMPLE_
%token SAOIMAGE_
%token SAOTNG_
%token SAVE_
%token SCAN_
%token SCIENTIFIC_
%token SCOPE_
%token SEGMENT_
%token SELECT_
%token SET_
%token SHARED_
%token SHIFT_
%token SHOW_
%token SIZE_
%token SLAVE_
%token SQRT_
%token SQUARE_
%token SQUARED_
%token STATUS_
%token SUM_
%token SYSTEM_
%token TEXT_
%token TO_
%token TRUE_
%token TYPE_
%token UNLOAD_
%token UNSELECT_
%token UPDATE_
%token USE_
%token USER_
%token VALUE_
%token VERSION_
%token VERTEX_
%token VERTICAL_
%token WARP_
%token WCS_
%token WIDTH_
%token X_
%token XY_
%token Y_
%token YES_
%token ZSCALE_
%token ZOOM_

%%

command : DEBUG_ debug

	| BLOCK_ block
	| CENTER_ {FR->centerCmd();}
	| CLEAR_ {FR->clearCmd();}
	| CLIP_ clip
	| COLORMAP_ colormap
	| COLORSCALE_ colorScale
	| CONTOUR_ contour
	| CROSSHAIR_ crosshair
	| GET_ get
	| HAS_ has
	| HIDE_ {FR->hideCmd();}
	| HIGHLITE_ yesno {FR->highliteCmd($2);}
	| IIS_ iis
	| LOAD_ load
	| MAGNIFIER_ magnifier
	| MARKER_ marker
	| ORIENT_ orient
	| PAN_ pan
	| PANNER_ panner
	| POSTSCRIPT_ postscript
	| RESET_ {FR->resetCmd();}
	| REGION_ region
	| ROTATE_ rotate
	| SCAN_ ON_ LOAD_ yesno	{FR->scanOnLoadCmd($4);}
	| SET_ set
	| SHOW_ {FR->showCmd();}
	| UNLOAD_ {FR->unloadFitsCmd();}
	| UPDATE_ {FR->updateFitsCmd();}
	| USE_ use
	| VERSION_ {FR->msg("Frame 1.0");}
	| WARP_ numeric numeric {FR->warpCmd(Vector($2,$3));}
	| WCS_ wcs
	| ZOOM_ zoom
	;

numeric	: REAL {$$=$1;}
	| INT {$$=$1;}
	;

debug	: ON_ {yydebug=1;}
	| OFF_ {yydebug=0;}
	;

yesno	: INT {$$=($1 ? 1 : 0);}

	| YES_ {$$=1;}
	| Y_ {$$=1;}
	| ON_ {$$=1;}
	| TRUE_ {$$=1;}

	| NO_ {$$=0;}
	| N_ {$$=0;}
	| OFF_ {$$=0;}
	| FALSE_ {$$=0;}
	;

precision : /* empty */  {$$ = Frame::DEFAULT;}
	| DEFAULT_ {$$ = Frame::DEFAULT;}
	| FIXED_ {$$ = Frame::FIXED;}
	| SCIENTIFIC_ {$$ = Frame::SCIENTIFIC;}
	| INTEGER_ {$$ = Frame::INTEGER;}
	;

angle	: /* empty */ {$$ = 0;}
	| numeric {$$ = degToRad($1);} /* assume degree */
	| DEGREE {$$ = degToRad($1);}
	| RADIAN {$$=$1;}
	;

coord	: hmsCoord dmsCoord {currentVector = Vector($1,$2);}
	| numeric numeric {currentVector = Vector($1,$2);}
	;

hmsCoord : HMSSTR {$$=parseHMS($1);}
	| HOUR MINUTE SECOND {$$=hmsToDegree(hmsSign,$1,$2,$3);}
	;

dmsCoord : HMSSTR {$$=parseDMS($1);}
	| numeric {hmsSign = ($1>=0?1:-1);} ARCMINUTE ARCSECOND
         {$$=dmsToDegree(hmsSign,fabs($1),$3,$4);}
	;

canvasCoord : /* empty */ {$$=Frame::PANNER;}
	| CANVAS_ {$$=Frame::CANVAS;}
	| PANNER_ {$$=Frame::PANNER;}
	;

coordSystemCanvas : /* empty */ {$$ = Frame::CANVAS;}
	| coordSystem {$$ = $1;}
	;

coordSystem : IMAGE_ {$$ = Frame::IMAGE;}
	| PHYSICAL_ {$$ = Frame::PHYSICAL;}
	| AMPLIFIER_ {$$ = Frame::AMPLIFIER;}
	| DETECTOR_ {$$ = Frame::DETECTOR;}
	| wcsSystem {$$ = $1;}
	;

coordFormat : /* empty */ {$$=Frame::PIXELS;}
	| PIXELS_ {$$=Frame::PIXELS;}
	| DEGREES_ {$$=Frame::DEGREES;}
	| ARCMIN_ {$$=Frame::ARCMIN;}
	| ARCSEC_ {$$=Frame::ARCSEC;}
	;

equatorial : B1950_  {$$ = Frame::EQUATORIAL_FK4;}
	| J2000_  {$$ = Frame::EQUATORIAL_FK5;}
	| FK4_  {$$ = Frame::EQUATORIAL_FK4;}
	| FK5_  {$$ = Frame::EQUATORIAL_FK5;}
	;

wcsSystem : EQUATORIAL_ equatorial {$$ = $2;}
	| FK4_ {$$ = Frame::EQUATORIAL_FK4;}
	| FK5_ {$$ = Frame::EQUATORIAL_FK5;}
	| B1950_ {$$ = Frame::EQUATORIAL_FK4;}
	| J2000_ {$$ = Frame::EQUATORIAL_FK5;}
	| GALACTIC_ {$$ = Frame::GALACTIC;}
	| ECLIPTIC_ {$$ = Frame::ECLIPTIC;}
	| LINEAR_ {$$ = Frame::LINEAR;}
	| ICRS_ {$$ = Frame::ICRS;}
	;

wcsFormat : DEGREES_ {$$ = Frame::DEGREES;}
	| HMS_ {$$ = Frame::HMS;}
	;

block	: FACTOR_ blockFactor
	| MODE_ blockMode 
	| FUNCTION_ blockFunction
	| BUFFER_ SIZE_ INT {FR->blockBufferSizeCmd($3);}
	| TO_ FIT_ {FR->blockToFitCmd();}
	;

blockFactor : numeric {FR->blockFactorCmd($1);}
	| numeric ABOUT_ coord coordSystemCanvas
	  {FR->blockFactorAboutCmd($1,currentVector,(Frame::CoordSystem)$4);}
	| TO_ numeric {FR->blockFactorToCmd($2);}
	| TO_ numeric ABOUT_ coord coordSystemCanvas
	  {FR->blockFactorToAboutCmd($2,currentVector,(Frame::CoordSystem)$5);}
	;

blockMode : AUTOMATIC_ {FR->blockModeCmd(Frame::AUTOMATICBLOCK);}
	| MANUAL_ {FR->blockModeCmd(Frame::MANUALBLOCK);}
	;

blockFunction : AVERAGE_ {FR->blockFunctionCmd(FitsHist::AVERAGE);}
	| SUM_ {FR->blockFunctionCmd(FitsHist::SUM);}
	;

clip	: SCOPE_ clipScope
	| MODE_ clipMode
	| MINMAX_ clipMinMax
	| USER_ clipUser
	| ZSCALE_ PARAM_ numeric INT INT {FR->clipZScaleParamCmd($3,$4,$5);}
	;

clipUser: /* empty */ numeric numeric {FR->clipUserCmd($1,$2);}
	| LOW_ numeric {FR->clipUserLowCmd($2);} 
	| HIGH_ numeric {FR->clipUserHighCmd($2);} 
	;

clipScope: GLOBAL_ {FR->clipScopeCmd(Frame::GLOBAL);}
	| LOCAL_ {FR->clipScopeCmd(Frame::LOCAL);}
	;

clipMode: MINMAX_ {FR->clipModeCmd(Frame::MINMAX);}
	| ZSCALE_ {FR->clipModeCmd(Frame::ZSCALE);}
	| USER_ {FR->clipModeCmd(Frame::USERCLIP);}
	;

clipMinMax : MODE_ clipMinMaxMode
	| PARAM_ INT {FR->clipMinMaxParamCmd($2);}
	;

clipMinMaxMode : SCAN_ {FR->clipMinMaxModeCmd(Frame::SCAN);}
	| SAMPLE_ {FR->clipMinMaxModeCmd(Frame::SAMPLE);}
	| DATAMIN_ {FR->clipMinMaxModeCmd(Frame::DATAMIN);}
	| IRAFMIN_ {FR->clipMinMaxModeCmd(Frame::IRAFMIN);}
	;

colormap : INT numeric numeric INT POINTER POINTER INT 
	  {FR->colormapCmd($1, $2, $3, $4, (unsigned short*)$5, 
	    (unsigned char*)$6, $7);}
	| BEGIN_ {FR->colormapBeginCmd();}
	| MOTION_ INT numeric numeric INT POINTER POINTER INT
	  {FR->colormapMotionCmd($2, $3, $4, $5, (unsigned short*)$6,
	    (unsigned char*)$7, $8);}
	| END_ INT numeric numeric INT POINTER POINTER INT
	  {FR->colormapEndCmd($2, $3, $4, $5, (unsigned short*)$6,
	    (unsigned char*)$7, $8);}
	;

colorScale: LINEAR_ {FR->colorScaleCmd(Frame::LINEARSCALE);}
	| LOG_ {FR->colorScaleCmd(Frame::LOGSCALE);}
	| SQUARED_ {FR->colorScaleCmd(Frame::SQUAREDSCALE);}
	| SQRT_ {FR->colorScaleCmd(Frame::SQRTSCALE);}
	;	

contour	: COLOR_ STRING {FR->contourColorCmd($2);}
	| COPY_ coordSystem {FR->contourCopyCmd((Frame::CoordSystem)$2);}
	| CREATE_ contourCreate
	| DELETE_ {FR->contourDeleteCmd();}
	| PASTE_ STRING POINTER coordSystem
	  {FR->contourPasteCmd($2, $3, (Frame::CoordSystem)$4);}
	;

contourCreate : STRING INT numeric {FR->contourCreateCmd($1,$2,$3);}
	| STRING INT numeric numeric numeric LINEAR_
	  {FR->contourCreateCmd($1,$2,$3,$4,$5,Frame::LINEARSCALE);}
	| STRING INT numeric numeric numeric LOG_
	  {FR->contourCreateCmd($1,$2,$3,$4,$5,Frame::LOGSCALE);}
	| STRING INT numeric numeric numeric SQUARED_
	  {FR->contourCreateCmd($1,$2,$3,$4,$5,Frame::SQUAREDSCALE);}
	| STRING INT numeric numeric numeric SQRT_
	  {FR->contourCreateCmd($1,$2,$3,$4,$5,Frame::SQRTSCALE);}
	;

crosshair: BEGIN_ MOTION_ numeric numeric coordSystemCanvas
	  {FR->crosshairBeginCmd(Vector($3,$4), (Frame::CoordSystem)$5);}
	| MOTION_ numeric numeric coordSystemCanvas
	  {FR->crosshairMotionCmd(Vector($2,$3), (Frame::CoordSystem)$4);}
	| coord coordSystemCanvas
	  {FR->crosshairCmd(currentVector, (Frame::CoordSystem)$2);}
	| yesno {FR->crosshairCmd($1);}
	| WARP_ numeric numeric {FR->crosshairWarpCmd(Vector($2,$3));}
	;

get	: AMPLIFIER_ COORDINATES_ canvasCoord numeric numeric precision
	  {FR->getAmplifierCoordCmd(Vector($4,$5),
	    (Frame::CoordSystem)$3,(Frame::Precision)$6);}
	| BLOCK_ getBlock
	| PHYSICAL_ COORDINATES_ canvasCoord numeric numeric precision
	  {FR->getPhysicalCoordCmd(Vector($4,$5),
	    (Frame::CoordSystem)$3,(Frame::Precision)$6);}
	| CLIP_ getClip
	| COLORMAP_ {FR->getColormapCmd();}
	| COLORSCALE_ {FR->getColorScaleCmd();}
	| CONTOUR_ COLOR_ {FR->getContourColorCmd();}
	| CROSSHAIR_ getCrosshair
	| CURSOR_ coordSystem precision 
	  {FR->getCursorCmd((Frame::CoordSystem)$2, (Frame::Precision)$3);}
	| DETECTOR_ COORDINATES_ canvasCoord numeric numeric precision
	{
	  FR->getDetectorCoordCmd(Vector($4,$5),
	    (Frame::CoordSystem)$3, (Frame::Precision)$6);
	}
	| FITS_ getFits
	| HEIGHT_ {FR->getHeightCmd();}
	| HISTOGRAM_ STRING STRING {FR->getHistogramCmd($2,$3);}
	| HORIZONTAL_ CUT_ getHorz
	| IIS_ getiis
	| IMAGE_ COORDINATES_ canvasCoord numeric numeric precision
	  {FR->getImageCoordCmd(Vector($4,$5),
	    (Frame::CoordSystem)$3,(Frame::Precision)$6);}
	| INFO_ canvasCoord numeric numeric wcsSystem wcsFormat STRING
	  {FR->getInfoCmd(Vector($3,$4), (Frame::CoordSystem)$2, 
	    (Frame::CoordSystem)$5, (Frame::CoordFormat)$6, $7);}
	| MINMAX_ canvasCoord numeric numeric
	  {FR->getMinMaxCmd(Vector($3,$4),(Frame::CoordSystem)$2);}
	| MARKER_ markerGet
	| NAME_ {FR->getNameCmd();}
	| ORIENT_ {FR->getOrientCmd();}
	| PAN_ coordSystem precision 
	  {FR->getPanCmd((Frame::CoordSystem)$2, (Frame::Precision)$3);}
	| ROTATE_ precision {FR->getRotateCmd((Frame::Precision)$2);}
	| SQUARE_ PIXELS_ {FR->getSquarePixelsCmd();}
	| VALUE_ canvasCoord numeric numeric
	  {FR->getValueCmd(Vector($3,$4),(Frame::CoordSystem)$2);}
	| VERTICAL_ CUT_ getVert
	| WCS_ getWCS
	| WIDTH_ {FR->getWidthCmd();}
	| ZOOM_ precision {FR->getZoomCmd((Frame::Precision)$2);}
	;

getBlock : FACTOR_ precision {FR->getBlockFactorCmd((Frame::Precision)$2);}
	| MODE_ {FR->getBlockModeCmd();}
	| FUNCTION_ {FR->getBlockFunctionCmd();}
	| BUFFER_ SIZE_ {FR->getBlockBufferSizeCmd();}
	| CURSOR_ precision {FR->getBlockCursorCmd((Frame::Precision)$2);}
	;

getClip	: precision {FR->getClipCmd((Frame::Precision)$1);}
	| canvasCoord numeric numeric precision
	  {FR->getClipCmd(Vector($2,$3),(Frame::CoordSystem)$1,
	    (Frame::Precision)$4);}
	| SCOPE_ {FR->getClipScopeCmd();}
	| MODE_ {FR->getClipModeCmd();}
	| MINMAX_ getClipMinMax
	| USER_ precision {FR->getClipUserCmd((Frame::Precision)$2);}
	| ZSCALE_ PARAM_ {FR->getClipZScaleParamCmd();}
	;

getClipMinMax : MODE_ {FR->getClipMinMaxModeCmd();}
	| PARAM_ {FR->getClipMinMaxParamCmd();}
	;

getCrosshair: /* empty */ {FR->getCrosshairCmd(Frame::CANVAS, Frame::DEFAULT);}
	| coordSystem precision 
	  {FR->getCrosshairCmd((Frame::CoordSystem)$1, (Frame::Precision)$2);}
	| STATUS_ {FR->getCrosshairStatusCmd();}
	;

getHorz	: STRING STRING {FR->getHorzGraphCmd($1,$2);}
	| STRING STRING numeric numeric 
	  {FR->getHorzGraphCmd($1,$2,Vector($3,$4));}
	;

getiis	: POINTER INT INT INT INT {FR->iisGetCmd((char*)$1,$2,$3,$4,$5);}
	| WCS_ {FR->iisGetWCSCmd();}
	;

getFits	: COUNT_ {FR->getFitsCountCmd();}
	| FILE_ NAME_ getFitsFileName
	| FILE_ NAMES_ {FR->getFitsFileNamesCmd(Frame::BASIC);}
	| FULL_ FILE_ NAME_ getFitsFullFileName
	| FULL_ FILE_ NAMES_ {FR->getFitsFileNamesCmd(Frame::FULL);}
	| HEADER_ INT {FR->getFitsHeaderCmd($2);}
	| OBJECT_ NAME_ getFitsObjectName
	| MASTER_ {FR->getFitsMasterCmd();}
	;

getFitsFileName: NUMBER_ INT {FR->getFitsFileNameCmd($2, Frame::BASIC);}
	| canvasCoord numeric numeric
	  {FR->getFitsFileNameCmd(Vector($2,$3),(Frame::CoordSystem)$1,
	    Frame::BASIC);}
	;

getFitsFullFileName: NUMBER_ INT {FR->getFitsFileNameCmd($2, Frame::FULL);}
	| canvasCoord numeric numeric
	  {FR->getFitsFileNameCmd(Vector($2,$3),(Frame::CoordSystem)$1,
	    Frame::FULL);}
	;

getFitsObjectName: NUMBER_ INT {FR->getFitsObjectNameCmd($2);}
	| canvasCoord numeric numeric
	  {FR->getFitsObjectNameCmd(Vector($2,$3), (Frame::CoordSystem)$1);}
	;

getVert	: STRING STRING {FR->getVertGraphCmd($1,$2);}
	| STRING STRING numeric numeric 
	  {FR->getVertGraphCmd($1,$2,Vector($3,$4));}
	;

getWCS	: ALIGN_ {FR->getWCSAlignCmd();}
	| ZOOM_ precision {FR->getWCSZoomCmd((Frame::Precision)$2);}
	| COORDINATES_ canvasCoord numeric numeric wcsSystem wcsFormat
	  {FR->getWCSCoordCmd(Vector($3,$4),(Frame::CoordSystem)$2,
	    (Frame::CoordSystem)$5,(Frame::CoordFormat)$6);}
	| SYSTEM_ {FR->getWCSSystemCmd();}
	;

has	: BLOCK_ {FR->hasBlockCmd();}
	| CONTOUR_ {FR->hasContourCmd();}
	| WCS_ {FR->hasWCSCmd();}
	| WCS_ EQUATORIAL_ {FR->hasWCSEquCmd();}
	| MOSAIC_ {FR->isMosaicCmd();}
	| PHYSICAL_ {FR->hasPhysicalCmd();}
	| DATAMIN_ {FR->hasDATAMINCmd();}
	| IRAFMIN_ {FR->hasIRAFMINCmd();}
	;

iis	: NEW_ INT INT {FR->iisCmd($2,$3);}
	| ERASE_ {FR->iisEraseCmd();}
	| SET_ POINTER INT INT INT INT
	  {FR->iisSetCmd((const char*)$2,$3,$4,$5,$6);}
	| WCS_ STRING numeric numeric numeric numeric 
	  numeric numeric numeric numeric numeric 
	    {FR->iisWCSCmd($2,Matrix($3,$4,$5,$6,$7,$8),Vector($9,$10),$11);}
	;

load	: STRING {FR->loadFitsMMapCmd($1);}
	| ARRAY_ loadArray
	| FITS_ loadFits
	| FITS_ MOSAIC_ loadMosaic
	| FITS_ MOSAIC_ EXT_ loadMosaicExt
	;

loadArray : STRING {FR->loadArrMMapCmd($1);}
	| ALLOC_ STRING {FR->loadArrAllocCmd($2);}
	| CHANNEL_ STRING {FR->loadArrChannelCmd($2, NULL);}
	| CHANNEL_ STRING STRING {FR->loadArrChannelCmd($2, $3);}
	| MMAP_ STRING {FR->loadArrMMapCmd($2);}
	| SHARED_ INT INT {FR->loadArrShareCmd($2, $3, NULL);}
	| SHARED_ INT INT STRING {FR->loadArrShareCmd($2, $3, $4);}
	;

loadFits : STRING {FR->loadFitsMMapCmd($1);}
	| ALLOC_ STRING {FR->loadFitsAllocCmd($2);}
	| CHANNEL_ STRING {FR->loadFitsChannelCmd($2, NULL);}
	| CHANNEL_ STRING STRING {FR->loadFitsChannelCmd($2, $3);}
	| MMAP_ STRING {FR->loadFitsMMapCmd($2);}
	| SHARED_ INT INT {FR->loadFitsShareCmd($2, $3, NULL);}
	| SHARED_ INT INT STRING {FR->loadFitsShareCmd($2, $3, $4);}
	| SLAVE_ STRING POINTER {FR->loadFitsSlaveCmd($2,(FitsFile*)$3);}
	;

loadMosaic : STRING {FR->loadMosaicMMapCmd($1);}
	| ALLOC_ STRING {FR->loadMosaicAllocCmd($2);}
	| CHANNEL_ STRING {FR->loadMosaicChannelCmd($2, NULL);}
	| CHANNEL_ STRING STRING {FR->loadMosaicChannelCmd($2, $3);}
	| MMAP_ STRING {FR->loadMosaicMMapCmd($2);}
	| SHARED_ INT INT {FR->loadMosaicShareCmd($2, $3, NULL);}
	| SHARED_ INT INT STRING {FR->loadMosaicShareCmd($2, $3, $4);}
	;

loadMosaicExt : STRING {FR->loadMosaicExtMMapCmd($1);}
	| ALLOC_ STRING {FR->loadMosaicExtAllocCmd($2);}
	| CHANNEL_ STRING {FR->loadMosaicExtChannelCmd($2, NULL);}
	| CHANNEL_ STRING STRING {FR->loadMosaicExtChannelCmd($2, $3);}
	| MMAP_ STRING {FR->loadMosaicExtMMapCmd($2);}
	| SHARED_ INT INT {FR->loadMosaicExtShareCmd($2, $3, NULL);}
	| SHARED_ INT INT STRING {FR->loadMosaicExtShareCmd($2, $3, $4);}
	;

magnifier: yesno {FR->magnifierCmd($1);}
	| GRAPHICS_ yesno {FR->magnifierGraphicsCmd($2);}
	| CURSOR_ yesno {FR->magnifierCursorCmd($2);}
	| MODE_ magnifierMode
	| STRING INT INT {FR->magnifierCmd($1, $2, $3);}
	| UPDATE_ numeric numeric {FR->updateMagnifierCmd(Vector($2, $3));}
	| ZOOM_ numeric {FR->magnifierZoomCmd($2);}
	;

magnifierMode: FAST_ {FR->magnifierModeCmd(Frame::FAST);}
	| ACCURATE_ {FR->magnifierModeCmd(Frame::ACCURATE);}
	;

marker	: COLOR_ STRING {FR->markerColorCmd($2);}
	| COMMAND_ {PASSTHRU_(0);} STRING {FR->markerCommandCmd($3);}
	| CREATE_ markerCreate
	| DELETE_ {FR->markerDeleteCmd();}
	| DELETE_ ALL_ {FR->markerDeleteAllCmd();}
	| EDIT_ markerEdit
	| FONT_ STRING {FR->markerFontCmd($2);}
	| INT CALLBACK_ markerCallBack STRING STRING
	  {FR->markerCallBackCmd($1,(Marker::Callback)$3,$4,$5);}
	| INT CIRCLE_ RADIUS_ numeric coordSystem coordFormat 
	  {FR->markerCircleRadiusCmd($1,$4,
	    (Frame::CoordSystem)$5,(Frame::CoordFormat)$6);}
	| INT COLOR_ STRING {FR->markerColorCmd($1,$3);}
	| INT CREATE_ POLYGON_ VERTEX_ INT numeric numeric
	  {FR->markerPolygonCreateVertexCmd($1,$5,Vector($6,$7));}
	| INT DELETE_ {FR->markerDeleteCmd($1);}
	| INT DELETE_ CALLBACK_ markerCallBack STRING
	  {FR->markerDeleteCallBackCmd($1,(Marker::Callback)$4,$5);}
	| INT DELETE_ POLYGON_ VERTEX_ INT 
	  {FR->markerPolygonDeleteVertexCmd($1,$5);}
	| INT EDIT_ BEGIN_ INT {FR->markerEditBeginCmd($1,$4);}
	| INT ELLIPSE_ RADIUS_ numeric numeric coordSystem coordFormat
	  {FR->markerEllipseRadiusCmd($1,Vector($4,$5),
	    (Frame::CoordSystem)$6,(Frame::CoordFormat)$7);}
	| INT ELLIPSE_ ANGLE_ angle {FR->markerEllipseAngleCmd($1,$4);}
	| INT FONT_ STRING {FR->markerFontCmd($1,$3);}
	| INT LINE_ ARROW_ yesno yesno {FR->markerLineArrowCmd($1,$4,$5);}
	| INT LINE_ POINT_ coord 
	  {currentVector2 = currentVector;} coord coordSystem
	  {FR->markerLineCmd($1, currentVector2, currentVector, 
	    (Frame::CoordSystem)$7);}
	| INT MOVE_ {currentID = $1} markerMoveID
	| INT PROPERTY_ markerProperty yesno {FR->markerPropertyCmd($1,$3,$4);}
	| INT POLYGON_ ANGLE_ angle {FR->markerPolygonAngleCmd($1,$4);}
	| INT RECTANGLE_ SIZE_ numeric numeric coordSystem coordFormat
	  {FR->markerRectangleSizeCmd($1,Vector($4,$5),
	    (Frame::CoordSystem)$6,(Frame::CoordFormat)$7);}
	| INT RECTANGLE_ ANGLE_ angle {FR->markerRectangleAngleCmd($1,$4);}
	| INT ROTATE_ BEGIN_ {FR->markerRotateBeginCmd($1);}
	| INT RULER_ POINT_ coord {currentVector2 = currentVector;} 
	  coord coordSystem
	  {FR->markerRulerPointCmd($1, currentVector2, currentVector,
	    (Frame::CoordSystem)$7);}
	| INT RULER_ SYSTEM_ coordSystem coordFormat
	  {FR->markerRulerSystemCmd($1, (Frame::CoordSystem)$4, 
	    (Frame::CoordFormat)$5);}
	| INT SELECT_ {FR->markerSelectCmd($1);}
	| INT TEXT_ STRING {FR->markerTextCmd($1,$3);}
	| INT UNSELECT_ {FR->markerUnselectCmd($1);}
	| LIST_ markerFileFormat coordSystem 
	  {FR->markerListCmd((Frame::MarkerFormat)$2,
	    (Frame::CoordSystem)$3,Frame::DEGREES);}
	| LIST_ markerFileFormat coordSystem wcsFormat
	  {FR->markerListCmd((Frame::MarkerFormat)$2,
	    (Frame::CoordSystem)$3,(Frame::CoordFormat)$4);}
	| LIST_ markerFileFormat coordSystem wcsFormat markerProperty yesno
	  {FR->markerListCmd((Frame::MarkerFormat)$2,
	    (Frame::CoordSystem)$3,(Frame::CoordFormat)$4,$5,$6);}
	| LOAD_ markerLoad
	| MOVE_ markerMoveSelected
	| PROPERTY_ markerProperty yesno {FR->markerPropertyCmd($2,$3);}
	| ROTATE_ BEGIN_ numeric numeric 
	  {FR->markerRotateBeginCmd(Vector($3,$4));}
	| ROTATE_ MOTION_ numeric numeric INT
	  {FR->markerRotateMotionCmd(Vector($3,$4),$5);}
	| ROTATE_ END_ {FR->markerRotateEndCmd();}
	| SAVE_ STRING markerFileFormat coordSystem 
	  {FR->markerSaveCmd($2,(Frame::MarkerFormat)$3,
	    (Frame::CoordSystem)$4,Frame::DEGREES);}
	| SAVE_ STRING markerFileFormat coordSystem wcsFormat
	  {FR->markerSaveCmd($2,(Frame::MarkerFormat)$3,
	    (Frame::CoordSystem)$4,(Frame::CoordFormat)$5);}
	| SELECT_ numeric numeric {FR->markerSelectCmd(Vector($2,$3));}
	| SELECT_ ALL_ {FR->markerSelectAllCmd();}
	| SELECT_ SHIFT_ numeric numeric 
	  {FR->markerSelectToggleCmd(Vector($3,$4));}
	| UNSELECT_ ALL_ {FR->markerUnselectAllCmd();}
	;

markerCallBack : SELECT_ {$$ = Marker::SELECTCB;}
	| EDIT_ {$$ = Marker::EDITCB;}
	| MOVE_  {$$ = Marker::MOVECB;}
	| ROTATE_ {$$ = Marker::ROTATECB;}
	| DELETE_ {$$ = Marker::DELETECB;}
	| TEXT_ {$$ = Marker::TEXTCB;}
	| COLOR_ {$$ = Marker::COLORCB;}
	| PROPERTY_ {$$ = Marker::PROPERTYCB;}
	| FONT_ {$$ = Marker::FONTCB;}
	;

markerLoad : STRING {FR->markerLoadCmd($1);}
	| INT {FR->markerLoadCmd($1);}
	| FITS_ STRING {FR->markerLoadFitsCmd($2);}
	;

markerProperty : NONE_ {$$ = Marker::NONE;}
	| SELECT_ {$$ = Marker::SELECT;}
	| EDIT_ {$$ = Marker::EDIT;}
	| MOVE_  {$$ = Marker::MOVE;}
	| ROTATE_ {$$ = Marker::ROTATE;}
	| DELETE_ {$$ = Marker::DELETE;}
	| INCLUDE_ {$$ = Marker::INCLUDE;}
	| FIXED_ {$$ = Marker::FIXED;}
	;

markerProperties : /* empty */ markerInitProp
	| markerInitProp markerProps
	;

markerInitProp : {
	  strncpy(currentFont, "system 10 normal", 32);
	  strncpy(currentColor, "green", 16);
	  strncpy(currentText, "", 80);
	  currentProps = defaultProps;
	}
	;

markerProps : markerProps markerProp
	| markerProp
	;

markerProp : markerProperty '=' yesno {setProps($1,$3);}
	| COLOR_ '=' STRING {strncpy(currentColor,$3,16);}
	| FONT_ '=' STRING {strncpy(currentFont,$3,32);}
	| TEXT_ '=' STRING {strncpy(currentText,$3,80);}
	;

markerCreate : CIRCLE_ coord coordSystemCanvas numeric 
	  coordFormat markerProperties
	  {FR->createCircleCmd(currentVector,(Frame::CoordSystem)$3, $4,
	    (Frame::CoordFormat)$5, currentColor,currentFont,
	      currentText,currentProps);}
	| ELLIPSE_ coord coordSystemCanvas numeric numeric coordFormat angle
	    markerProperties
	  {FR->createEllipseCmd(currentVector,(Frame::CoordSystem)$3,
	    Vector($4,$5), (Frame::CoordFormat)$6,$7,
	    currentColor,currentFont,currentText,currentProps);}
	| RECTANGLE_ coord coordSystemCanvas numeric numeric coordFormat angle
	    markerProperties
	  {FR->createRectangleCmd(currentVector,(Frame::CoordSystem)$3,
	    Vector($4,$5), (Frame::CoordFormat)$6,$7,
	    currentColor,currentFont,currentText,currentProps);}
	| POLYGON_ coord numeric numeric coordSystemCanvas markerProperties
	  {FR->createPolygonCmd(currentVector,Vector($3,$4),
	    (Frame::CoordSystem)$5, currentColor,currentFont,
	    currentText,currentProps);}
	| LINE_ coord {currentVector2 = currentVector;} 
	  coord coordSystemCanvas markerProperties
	  {FR->createLineCmd(currentVector2,currentVector,
	    (Frame::CoordSystem)$5,0,0,currentColor,currentFont,
	    currentText,currentProps);}
	| TEXT_ coord coordSystemCanvas markerProperties
	  {FR->createTextCmd(currentVector,(Frame::CoordSystem)$3,
	    currentColor,currentFont,currentText,currentProps);}
	| RULER_ coord {currentVector2 = currentVector;} coord 
	    coordSystemCanvas coordFormat markerProperties
	  {FR->createRulerCmd(currentVector2,currentVector,
	    (Frame::CoordSystem)$5,(Frame::CoordFormat)$6,
	    currentColor,currentFont,currentText,currentProps);}
	| CIRCLE_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createCirclePointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	| BOX_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createBoxPointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	| DIAMOND_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createDiamondPointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	| CROSS_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createCrossPointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	| X_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createExPointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	| ARROW_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createArrowPointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	| BOXCIRCLE_ POINT_ coord coordSystemCanvas markerProperties
	  {FR->createBoxCirclePointCmd(currentVector,(Frame::CoordSystem)$4,
	    currentColor,currentFont,currentText,currentProps);}
	;

markerEdit : BEGIN_ numeric numeric INT 
	  {FR->markerEditBeginCmd(Vector($2,$3),$4);}
	| MOTION_ numeric numeric INT 
	  {FR->markerEditMotionCmd(Vector($2,$3),$4);}
	| END_ {FR->markerEditEndCmd();}
	;

markerFileFormat : DS9_ {$$ = Frame::DS9;}
	| SAOTNG_ {$$ = Frame::SAOTNG;}
	| SAOIMAGE_ {$$ = Frame::SAOIMAGE;}
	| PROS_ {$$ = Frame::PROS;}
	;

markerGet : 
	| COLOR_ {FR->getMarkerColorCmd();}
	| FONT_ {FR->getMarkerFontCmd();}
	| HANDLE_ numeric numeric {FR->getMarkerHandleCmd(Vector($2,$3));}
	| ID_ numeric numeric {FR->getMarkerIdCmd(Vector($2,$3));}
	| ID_ ALL_ {FR->getMarkerIdAllCmd();}
	| INT CENTER_ coordSystem 
	  {FR->getMarkerCenterCmd($1,(Frame::CoordSystem)$3);}
	| INT CENTER_ wcsSystem wcsFormat 
	  {FR->getMarkerCenterCmd($1,(Frame::CoordSystem)$3,
	  (Frame::CoordFormat)$4);}
	| INT CIRCLE_ RADIUS_ coordSystem coordFormat
	  {FR->getMarkerCircleRadiusCmd($1,
	    (Frame::CoordSystem)$4,(Frame::CoordFormat)$5);}
	| INT COLOR_ {FR->getMarkerColorCmd($1);}
	| INT ELLIPSE_ RADIUS_ coordSystem coordFormat
	  {FR->getMarkerEllipseRadiusCmd($1,
	    (Frame::CoordSystem)$4,(Frame::CoordFormat)$5);}
	| INT ELLIPSE_ ANGLE_ {FR->getMarkerEllipseAngleCmd($1);}
	| INT FONT_ {FR->getMarkerFontCmd($1);}
	| INT LINE_ ARROW_ {FR->getMarkerLineArrowCmd($1);}
	| INT LINE_ POINT_ coordSystem 
	  {FR->getMarkerLineCmd($1,(Frame::CoordSystem)$4);}
	| INT LINE_ POINT_ wcsSystem wcsFormat
	  {FR->getMarkerLineCmd($1, (Frame::CoordSystem)$4, 
	    (Frame::CoordFormat)$5);}
	| INT PROPERTY_ {FR->getMarkerPropertyCmd($1);}
	| INT PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($1,$3);}
	| INT POLYGON_ ANGLE_ {FR->getMarkerPolygonAngleCmd($1);}
	| INT RECTANGLE_ SIZE_ coordSystem coordFormat
	  {FR->getMarkerRectangleSizeCmd($1,
	    (Frame::CoordSystem)$4,(Frame::CoordFormat)$5);}
	| INT RECTANGLE_ ANGLE_ {FR->getMarkerRectangleAngleCmd($1);}
	| INT RULER_ DISTANCE_ {FR->getMarkerRulerDistanceCmd($1);}
	| INT RULER_ POINT_ coordSystem
	  {FR->getMarkerRulerPointCmd($1,(Frame::CoordSystem)$4);}
	| INT RULER_ POINT_ wcsSystem wcsFormat
	  {FR->getMarkerRulerPointCmd($1, (Frame::CoordSystem)$4,
	    (Frame::CoordFormat)$5);}
	| INT TEXT_ {FR->getMarkerTextCmd($1);}
	| INT TYPE_ {FR->getMarkerTypeCmd($1);}
	| POLYGON_ SEGMENT_ numeric numeric
	  {FR->getMarkerPolygonSegmentCmd(Vector($3,$4));}
	| PROPERTY_ markerProperty {FR->getMarkerPropertyCmd($2);}
	| SELECT_ markergetselect
	| SELECT_ NUMBER_ {FR->getMarkerSelectedNumberCmd();}
	;

markergetselect : /* empty */ {FR->getMarkerSelectedCmd();}
	| numeric numeric {FR->getMarkerSelectedCmd(Vector($1,$2));}
	;

markerMoveID : numeric numeric {FR->markerMoveCmd(currentID, Vector($1,$2));}
	| FRONT_ {FR->markerFrontCmd(currentID);}
	| BACK_ {FR->markerBackCmd(currentID);}
	| BEGIN_ numeric numeric 
	  {FR->markerMoveBeginCmd(currentID, Vector($2,$3));}
	| MOTION_ numeric numeric 
	  {FR->markerMoveMotionCmd(currentID, Vector($2,$3));}
	| END_ {FR->markerMoveEndCmd(currentID);}
	| TO_ coord coordSystem
	  {FR->markerMoveToCmd(currentID, currentVector, 
	  (Frame::CoordSystem)$3);}
	;

markerMoveSelected : numeric numeric {FR->markerMoveCmd(Vector($1,$2));}
	| FRONT_ {FR->markerFrontCmd();}
	| BACK_ {FR->markerBackCmd();}
	| BEGIN_ numeric numeric {FR->markerMoveBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {FR->markerMoveMotionCmd(Vector($2,$3));}
	| END_ {FR->markerMoveEndCmd();}
	| TO_ coord coordSystem
	  {FR->markerMoveToCmd(currentVector, (Frame::CoordSystem)$3);}
	;

orient	: X_ {FR->orientCmd(Frame::X);}
	| Y_ {FR->orientCmd(Frame::Y);}
	| XY_ {FR->orientCmd(Frame::XY);}
	| NONE_ {FR->orientCmd(Frame::NORMAL);}
	;

pan	: numeric numeric numeric numeric 
	  {FR->panCmd(Vector($1,$2),Vector($3,$4));}
	| coord coordSystemCanvas 
	  {FR->panCmd(currentVector,(Frame::CoordSystem)$2);}
	| TO_ coord coordSystemCanvas
	  {FR->panToCmd(currentVector,(Frame::CoordSystem)$3);}
	| BBOX_ numeric numeric numeric numeric precision
	  {FR->panBBoxCmd(BBox($2,$3,$4,$5), (Frame::Precision)$6);}
	| MOTION_ panMotion
	;

panMotion : BEGIN_ numeric numeric {FR->panMotionBeginCmd(Vector($2,$3));}
	| numeric numeric {FR->panMotionCmd(Vector($1,$2));}
	| END_ numeric numeric {FR->panMotionEndCmd(Vector($2,$3));}
	;

panner	: yesno {FR->pannerCmd($1);}
	| STRING INT INT {FR->pannerCmd($1, $2, $3);}
	| UPDATE_ {FR->updatePannerCmd();}
	;

postscript : COLORSPACE_ pscolorspace 
	  {FR->psColorSpaceCmd((Frame::PSColorSpace)$2);}
	| INTERPOLATE_ yesno {FR->psInterpolateCmd($2);}
	| LEVEL_ INT {FR->psLevelCmd($2);}
	| RESOLUTION_ INT {FR->psResolutionCmd($2);}
	;

pscolorspace : BW_ {$$ = Frame::BW;}
	| GRAY_ {$$ = Frame::GRAY;}
	| RGB_ {$$ = Frame::RGB;}
	| CMYK_ {$$ = Frame::CMYK;}
	;

region	: SELECT_ regionSelect
	;

regionSelect : BEGIN_ numeric numeric 
	  {FR->regionSelectBeginCmd(Vector($2,$3));}
	| MOTION_ numeric numeric {FR->regionSelectMotionCmd(Vector($2,$3));}
	| END_ {FR->regionSelectEndCmd();}
	| SHIFT_ END_ {FR->regionSelectShiftEndCmd();}
	;

rotate	: numeric {FR->rotateCmd(degToRad($1));}
	| numeric DEGREES_ {FR->rotateCmd(degToRad($1));}
	| MOTION_ rotateMotion
	| TO_ numeric {FR->rotateToCmd(degToRad($2));}
	| TO_ numeric DEGREES_ {FR->rotateToCmd(degToRad($2));}
	;

rotateMotion : BEGIN_ {FR->rotateMotionBeginCmd();}
	| numeric {FR->rotateMotionCmd(degToRad($1));}
	| END_ {FR->rotateMotionEndCmd();}
	;

set	: NAME_ STRING {FR->setNameCmd($2);}
	;

use	: SQUARE_ PIXELS_ yesno {FR->setSquarePixelsCmd($3);}
	;

wcs	: ALIGN_ yesno {FR->wcsAlignCmd($2);}
	| ZOOM_ numeric {FR->wcsZoomCmd($2);}
	;

zoom	: numeric {FR->zoomCmd($1);}
	| numeric ABOUT_ coord coordSystemCanvas
	  {FR->zoomAboutCmd($1,currentVector,(Frame::CoordSystem)$4);}
	| TO_ zoomTo
	;

zoomTo	: FIT_	{FR->zoomToFitCmd();}
	| numeric {FR->zoomToCmd($1);}
	| numeric ABOUT_ coord coordSystemCanvas
	  {FR->zoomToAboutCmd($1,currentVector,(Frame::CoordSystem)$4);}
	;

%%

static void setProps(unsigned short prop, int value)
{
  if (value)
    currentProps |= prop;
  else
    currentProps &= ~prop;
}
