#ifndef __ps_h__
#define __ps_h__

#include <iostream.h>

// Filter Base Class

class Filter {
protected:
  unsigned char buf[256];
  unsigned char* ptr;

  virtual void cflush() =0;
  virtual void eflush(ostream&) =0;

public:
  Filter();

  virtual void in(unsigned char) =0;
  virtual void out(ostream&) =0;
  void flush(ostream&);

  friend Filter& operator<<(Filter& f, unsigned char c) {f.in(c); return f;}
  friend ostream& operator<<(ostream& s, Filter& f) {f.out(s); return s;}
};

// Compress Base Class

class Compress : public virtual Filter {
public:
  virtual void in(unsigned char) =0;
  virtual void cflush() {}
};

class NoCompress : public virtual Filter, public Compress {
public:
  void in(unsigned char);
};

class RLE : public virtual Filter, public Compress {
private:
  int state;
  unsigned char current;
  unsigned char rle[128];
  int num;

  void dumpNonRepeat();
  void dumpRepeat();

public:
  RLE();

  void in(unsigned char);
  void cflush();
};

// Encode Base Class

#define LINELIMIT 80

class Encode : public virtual Filter {
protected:
  int lineCount;

public:
  Encode();

  virtual void out(ostream&) =0;
  virtual void eflush(ostream&) =0;
};

class AsciiHex : public virtual Filter, public Encode {
public:
  void out(ostream&);
  void eflush(ostream&);
};

class Ascii85 : public virtual Filter, public Encode {
private:
  int index;
  int byteswap;
  union {
    unsigned char b[4];
    unsigned long c;
  } buf85;

  void dump(ostream&);
  long swap(unsigned long* ptr);
  void swap4(char* src, char* dest)
    {*dest = *(src+3); *(dest+1) = *(src+2); *(dest+2) = *(src+1); *(dest+3) = *src;}

public:
  Ascii85();

  void out(ostream&);
  void eflush(ostream&);
};

// PS Filters

class NoCompressAsciiHex : public virtual Filter, public NoCompress, public AsciiHex {};
class NoCompressAscii85 : public virtual Filter, public NoCompress, public Ascii85 {};
class RLEAsciiHex : public virtual Filter, public RLE, public AsciiHex {};
class RLEAscii85 : public virtual Filter, public RLE, public Ascii85 {};

#endif

