/*
 *
 * stest -- server test for xpa
 *
 */

/*
 *      Copyrights:
 *
 *    	  Copyright (c) 1996, 1998 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifdef HAVE_XT
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#endif
#ifdef HAVE_TCL
#include <tcl.h>
#ifdef USE_TK
#include <tk.h>
#endif
#endif
#include <xpap.h>
#include <word.h>

#define MAX_FPS 10

extern char *optarg;
extern int optind;

int  quiet=0;
int  dowait=0;
int  save_bytes=-1;
int n=0;
char *save_buf=NULL;

char *mode="";
char name[SZ_LINE];
char xclass[SZ_LINE];

#ifdef ANSI_FUNC
int 
send_cb (void *client_data, void *call_data, char *paramlist,
	 char **buf, int *len)
#else
int send_cb(client_data, call_data, paramlist, buf, len)
     void *client_data;
     void *call_data;
     char *paramlist;
     char **buf;
     int *len;
#endif
{
  char *s = (char *)client_data;
  XPA xpa = (XPA)call_data;
  char tbuf[SZ_LINE];
  int sendbuf=0;

  /* introduce ourselves */
  if( !quiet ){
    fprintf(stderr, "SEND_CB #%d: %s:%s (%s)\n",
	    n++, xpa_class(xpa), xpa_name(xpa), s);
  }

  /* process special paramlist tokens */
  if( paramlist && *paramlist ){
    if( !quiet )
      fprintf(stderr, "\tparamlist: %s\n", paramlist);
    if( !strncmp(paramlist, "buf", 3) ){
      sendbuf=1;
    }
    else if( !strncmp(paramlist, "error", 5) ){
      if( !quiet )
	fprintf(stderr, "\treturning error: %s\n", &paramlist[6]);
      *len = 0;
      *buf = NULL;
      XPAError(xpa, &paramlist[6]);
      return(-1);
    }
    else if( !strcmp(paramlist, "exit") ){
      if( !quiet )
	fprintf(stderr, "Exiting\n");
      exit(0);
    }
    else if( !strcmp(paramlist, "wait") ){
      fprintf(stderr, "Press <CR> to continue ...");
      gets(tbuf);
    }
  }
  else if( dowait ){
    fprintf(stderr, "Press <CR> to continue ...");
    gets(tbuf);
  }

  /* return information about this xpa */
  if( !sendbuf ){
    sprintf(tbuf,
	  "class: %s\nname: %s\nmethod: %s\nsendian: %s\ncendian: %s\n",
	  xpa_class(xpa), xpa_name(xpa), xpa_method(xpa),
	  xpa_sendian(xpa), xpa_cendian(xpa));
  
    if( (xpa->send_mode & XPA_MODE_FILLBUF) ){
      write(xpa_datafd(xpa), tbuf, strlen(tbuf));
      *len = 0;
      *buf = NULL;
      if( !quiet)
	fprintf(stderr, "\tcallback writes %d bytes to client\n",
		strlen(tbuf));
    }
    /* return the buffer and let xpa transmit it */
    else{
      *len = strlen(tbuf);
      *buf = (char *)malloc(*len);
      memcpy(*buf, tbuf, *len);
      if( !quiet)
	fprintf(stderr, "\tcallback returns %d bytes to xpa handler\n",
		strlen(tbuf));
    }
  }
  /* return the last buffer we were sent */
  else{
    if( (xpa->send_mode & XPA_MODE_FILLBUF) ){
      write(xpa_datafd(xpa), save_buf, save_bytes);
      *len = 0;
      *buf = NULL;
      if( !quiet)
	fprintf(stderr, "\tcallback writes %d bytes to client\n", save_bytes);
    }
    /* return the buffer and let xpa transmit it */
    else{
      *len = save_bytes;
      *buf = (char *)malloc(*len);
      memcpy(*buf, save_buf, *len);
      if( !quiet)
	fprintf(stderr, "\tcallback returns %d bytes to xpa handler\n",
		save_bytes);
    }
  }
  if( !quiet ){
    fprintf(stderr, "SEND_CB complete\n");
  }

  return(0);
}

#ifdef ANSI_FUNC
int 
receive_cb (void *client_data, void *call_data, char *paramlist,
	    char *buf, int len)
#else
int receive_cb(client_data, call_data, paramlist, buf, len)
     void *client_data;
     void *call_data;
     char *paramlist;
     char *buf;
     int len;
#endif
{
  XPA xpa = (XPA)call_data;
  char *s = (char *)client_data;
  char tbuf[SZ_LINE];
  char cbuf[SZ_LINE];
  char *errs[1];
  int i;
  int ip;
  int got;
  int xwait;

  if( !quiet ){
    fprintf(stderr, "RECEIVE_CB #%d: %s:%s (%s)\n",
	    n++, xpa_class(xpa), xpa_name(xpa), s);
  }
  /* process param list */
  if( paramlist && *paramlist ){
    if( !quiet )
      fprintf(stderr, "\tparamlist: %s\n", paramlist);
    if( !strcmp(paramlist, "free") ){
      if( !quiet )
	fprintf(stderr, "Freeing xpa struct\n");
      XPAFree(xpa);
      return(0);
    }
    else if( !strncmp(paramlist, "error", 5) ){
      if( !quiet )
	fprintf(stderr, "Processing error: %s\n", &paramlist[6]);
      XPAError(xpa, &paramlist[6]);
      return(-1);
    }
    else if( !strcmp(paramlist, "exit") ){
      if( !quiet )
	fprintf(stderr, "Exiting\n");
      exit(0);
    }
    else if( !strcmp(paramlist, "wait") ){
      fprintf(stderr, "Press <CR> to continue ...");
      gets(tbuf);
    }
    else if( !strncmp(paramlist, "xpaset", 6) ){
      ip = 0;
      word(paramlist, tbuf, &ip);
      if( word(paramlist, tbuf, &ip) ){
	if( !quiet )
	  fprintf(stderr, "calling XPASet(%s, \"%s\")\n",
		  tbuf, &(paramlist[ip]));
	got = XPASet(NULL, tbuf, &(paramlist[ip]), mode,
		     paramlist, strlen(paramlist), NULL, errs, 1);
	if( got == 0 ){
	  if( !quiet )
	    fprintf(stderr, "no XPA access points matched template %s\n",
		    tbuf);
	}
	else if( errs[0] != NULL ){
	  if( !quiet )
	    fprintf(stderr, "Error on xpaset to %s: %s\n", tbuf, errs[0]);
	  free(errs[0]);
	}
	else{
	  if( !quiet )
	    fprintf(stderr, "XPASet to %s successful\n", tbuf);
	}
      }
      return(0);
    }
  }
  else if( dowait ){
    fprintf(stderr, "Press <CR> to continue ...");
    gets(tbuf);
  }

  /* reset save buffer */
  if( save_buf != NULL ){
    free(save_buf);
    save_buf = NULL;
  }
  save_bytes = 0;

  xwait = dowait;
  if( !(xpa->receive_mode & XPA_MODE_FILLBUF) ){
    while( (got=read(xpa_datafd(xpa), tbuf, SZ_LINE)) >0 ){
      if( xwait >0 ){
	fprintf(stderr, "got %d bytes ... press <CR> to continue ...", got);
	gets(cbuf);
	xwait--;
      }
      i = save_bytes;
      save_bytes += got;
      if( save_buf == NULL )
	save_buf = (char *)malloc(save_bytes);
      else
	save_buf = (char *)realloc(save_buf, save_bytes);
      memcpy(&save_buf[i], tbuf, got);
    }
    if( !quiet )
      fprintf(stderr, "\tcallback read %d bytes\n", save_bytes);
  }
  else{
    if( !quiet ){
      fprintf(stderr, "\tenter callback with buf: %d bytes\n", len);
    }
    save_bytes = len;
    save_buf = (char *)malloc(len);
    memcpy(save_buf, buf, len);
  }
  if( !quiet ){
    fprintf(stderr, "RECEIVE_CB complete\n");
  }
  return(0);
}

#ifdef ANSI_FUNC
int 
info_cb (void *client_data, void *call_data, char *paramlist)
#else
int info_cb(client_data, call_data, paramlist)
     void *client_data;
     void *call_data;
     char *paramlist;
#endif
{
  XPA xpa = (XPA)call_data;
  char *s = (char *)client_data;
  char tbuf[SZ_LINE];
  char xtemplate[SZ_LINE];
  FILE *fd;
  char *names[MAX_FPS];
  char *bufs[MAX_FPS];
  char *errs[MAX_FPS];
  int lens[MAX_FPS];
  int i;
  int ip;
  int got;

  if( !quiet ){
    fprintf(stderr, "INFO_CB #%d: %s:%s (%s)\n",
	    n++, xpa_class(xpa), xpa_name(xpa), s);
  }

  if( paramlist && *paramlist ){
    if( !quiet )
      fprintf(stderr, "\tparamlist: %s\n", paramlist);
    ip = 0;
    word(paramlist, xtemplate, &ip);
    if( !strcmp(xtemplate, "xpaget") ){
      word(paramlist, xtemplate, &ip);
      got = XPAGet(xpa, xtemplate, &(paramlist[ip]), NULL,
		   bufs, lens, names, errs, MAX_FPS);
      if( !quiet )
	fprintf(stderr, "XPAGet (%s) returned %d buffers\n", xtemplate, got);
      for(i=0; i<got; i++){
	if( !quiet )
	  fprintf(stderr, "\t%d: %s\n", i, names[i]);
	if( errs[i] == NULL ){
	  if( lens[i] > 0 ){
	    write(fileno(stderr), bufs[i], lens[i]);
	  }
	  if( !quiet )
	    fprintf(stderr, "\n");
	}
	else{
	  write(fileno(stderr), errs[i], strlen(errs[i]));
	}
	if( bufs[i] )
	  free(bufs[i]);
	if( names[i] )
	  free(names[i]);
	if( errs[i] )
	  free(errs[i]);
      }
    }
  }
  if( !quiet ){
    fprintf(stderr, "INFO_CB complete\n");
  }
  return(0);
}

#ifdef HAVE_TCL
#ifdef USE_TK
#ifdef ANSI_FUNC
int 
Tk_AppInit(Tcl_Interp *interp)
#else
int Tk_AppInit(interp)
     Tcl_Interp *interp;
#endif
{
  if( Tcl_Init(interp) == TCL_ERROR )
    return(TCL_ERROR);

  if( Tk_Init(interp) == TCL_ERROR )
    return(TCL_ERROR);

  return(TCL_OK);
}
#else
#ifdef ANSI_FUNC
int 
Tcl_AppInit(Tcl_Interp *interp)
#else
int Tcl_AppInit(interp)
     Tcl_Interp *interp;
#endif
{
  if( Tcl_Init(interp) == TCL_ERROR )
    return(TCL_ERROR);

  return(TCL_OK);
}
#endif
#endif

#ifdef ANSI_FUNC
int 
main (int argc, char **argv)
#else
main(argc, argv)
     int argc;
     char **argv;
#endif
{
  XPA xpa1, xpa1a, xpa2, xpa3, xpa4;
#ifdef HAVE_XT
  Widget top;
  XtAppContext appcontext;
#endif
#ifdef HAVE_TCL  
  Tcl_Interp *interp;
#endif
  char tbuf[SZ_LINE];
  char tbuf2[SZ_LINE];
  char cmd[SZ_LINE];
  int c;
  int x=0, tcl=0;
  int delay=0;

  *cmd = '\0';
  /* process switch arguments */
  while ((c = getopt(argc, argv, "ad:f:qrstuwx")) != -1){
    switch(c){
    case 'a':
      mode = "ack=false";
      break;
    case 'd':
      delay = atoi(optarg);
      break;
    case 'f':
      sprintf(cmd, "%s %s &\n", argv[0], optarg);
      break;
    case 'q':
      quiet = 1;
      break;
    case 'r':
      mode = "fillbuf=false";
      break;
    case 's':
      x = 0;
      tcl = 0;
      break;
    case 't':
      x = 0;
      tcl = 1;
      break;
    case 'u':
      mode = "buf=false";
      break;
    case 'w':
      dowait++;
      break;
    case 'x':
      x = 1;
      tcl = 0;
      break;
    }
  }

  if( optind >= argc ){
    strcpy(name, "xpa");
  }
  else{
    strcpy(name, argv[optind]);
  }
  strcpy(xclass, name);
  cluc(xclass);

  if( x ){
#ifdef HAVE_XT
    top = XtAppInitialize(&appcontext, "Server", NULL, 0,
			  &argc, argv, NULL, NULL, 0);
    XtVaSetValues(top, XtNwidth, (XtArgVal)10, XtNheight, (XtArgVal)10,
		  XtNmappedWhenManaged, (XtArgVal)False, NULL);
#else
    fprintf(stderr, "Xt is not available ...\n");
    exit(1);
#endif
  }

  if( tcl ){
#ifndef HAVE_TCL
    fprintf(stderr, "Tcl is not available ...\n");
    exit(1);
#endif
  }

  if( !strcmp(name, "help") )
    strcpy(tbuf, "help");
  else
    sprintf(tbuf, "%s1", name);
  sprintf(tbuf2, "help for %s", tbuf);
  xpa1 = XPANew(xclass, tbuf, tbuf2,
		send_cb, "send1", mode, receive_cb, "receive1", mode);
  if( xpa1 == NULL ){
    fprintf(stderr, "ERROR: could not init xpa1\n");
  }
  else{
    fprintf(stderr, "%s using method: %s\n", xpa_name(xpa1), xpa_method(xpa1));
  }

  sprintf(tbuf, "%s1a", name);
  sprintf(tbuf2, "help for %s", tbuf);
  xpa1a = XPANew(xclass, tbuf, tbuf2,
	send_cb, "send1a", mode, receive_cb, "receive1a", mode);
  if( xpa1a == NULL ){
    fprintf(stderr, "ERROR: could not init xpa1a\n");
  }
  else{
    fprintf(stderr, "%s using method: %s\n", 
	    xpa_name(xpa1a), xpa_method(xpa1a));
  }

  sprintf(tbuf, "%sc", name);
  xpa2 = XPACmdNew(xclass, tbuf);
  if( xpa2 == NULL ){
    fprintf(stderr, "ERROR: could not init xpa2\n");
  }
  else{
    XPACmdAdd(xpa2, "cmd2", "and help for cmd2",
	      send_cb, "cmd2", mode, receive_cb, "cmd2", mode);
    XPACmdAdd(xpa2, "cmd1 xx yy", "help for cmd1 xx yy",
	      send_cb, "cmd1 xx yy", mode, receive_cb, "cmd1 xx yy", mode);
    XPACmdAdd(xpa2, "cmd1 xx", "help for cmd1 xx",
	      send_cb, "cmd1 xx", mode, receive_cb, "cmd1 xx", mode);
    XPACmdAdd(xpa2, "cmd1", NULL,
	      send_cb, "cmd1", mode, receive_cb, "cmd1", mode);
    fprintf(stderr, "%s using method: %s\n", xpa_name(xpa2), xpa_method(xpa2));
  }

  sprintf(tbuf, "%si", name);
  xpa3 = XPAInfoNew(xclass, tbuf, info_cb, "info1", mode);
  if( xpa3 == NULL ){
    fprintf(stderr, "ERROR: could not init xpa3\n");
  }
  else{
    fprintf(stderr, "%s using method: %s\n", xpa_name(xpa3), xpa_method(xpa3));
  }

  if( *cmd != '\0' ){
    fprintf(stderr, "starting bkgd process: %s", cmd);
    system(cmd);
  }

  /* delay if necessary */
  if( delay ){
    fprintf(stderr, "starting delay of %d seconds ...", delay);
    sleep(delay);
    fprintf(stderr, " done\n");
  }

  if( x ){
#ifdef HAVE_XT
    fprintf(stderr, "\nEntering Xt loop ...\n");
    XPAXtAddInput(appcontext, NULL);
    XtRealizeWidget(top);
    XtAppMainLoop(appcontext);
#else
    fprintf(stderr, "Xt is not available ...\n");
    exit(1);
#endif
  }
  else if( tcl ){
#ifdef HAVE_TCL
    XPATclAddInput(NULL);
#ifdef USE_TK
    fprintf(stderr, "\nEntering Tk loop ...\n");
    Tk_Main(1, argv, Tk_AppInit);
#else
    fprintf(stderr, "\nEntering Tcl loop ...\n");
    Tcl_Main(argc, argv, Tcl_AppInit);
#endif
#else
    fprintf(stderr, "Tcl is not available ...\n");
    exit(1);
#endif
  }
  else{
    fprintf(stderr, "\nEntering select loop ...\n");
    XPAMainLoop();
  }
  exit(0);
}
